@@ -36,6 +36,8 @@ typedef struct r2r_state_t {
 } R2RState;
 
 static RThreadFunctionRet worker_th(RThread *th);
+/* Mutex to serialize multi-line failure printing across threads */
+static RThreadLock *r2r_print_lock = NULL;
 static void print_state(R2RState *state, ut64 prev_completed);
 static void print_log(R2RState *state, ut64 prev_completed, ut64 prev_paths_completed);
 static void interact(R2RState *state);
@@ -84,6 +86,7 @@ static void helpvars(int workers_count) {
 		"R2R_TIMEOUT=%d   # timeout after 1 minute (60 * 60)\n"
 		"R2R_OFFLINE=0      # same as passing -u\n"
 		"R2R_SHALLOW=0      # skip 0-100%% random tests\n"
+		"R2R_RADARE2=radare2 # radare2 binary to launch\n"
 		, workers_count, TIMEOUT_DEFAULT
 	       );
 }
@@ -472,6 +475,9 @@ int main(int argc, char **argv) {
 	}
 	atexit (r2r_subprocess_fini);
 
+	/* print lock for atomic multi-line output */
+	r2r_print_lock = r_th_lock_new (false);
+
 	char *have_options = r_sys_getenv ("ASAN_OPTIONS");
 	if (have_options) {
 		free (have_options);
@@ -485,6 +491,11 @@ int main(int argc, char **argv) {
 	ut64 time_start = r_time_now_mono ();
 	R2RState state = {{0}};
 	state.run_config.r2_cmd = "radare2";
+	char *r2_cmd = r_sys_getenv("R2R_RADARE2");
+	if (R_STR_ISNOTEMPTY(r2_cmd)) {
+		R_LOG_DEBUG("overriding radare2 command: r2_cmd=%s", r2_cmd);
+		state.run_config.r2_cmd = r2_cmd;
+	}
 	if (shallow > 0) {
 		r_num_irand ();
 		state.run_config.shallow = shallow;
@@ -974,34 +985,39 @@ R_API bool r_test_cmp_cmd_output(const char *output, const char *expect, const c
 }
 
 static void print_result_diff(R2RRunConfig *config, R2RTestResultInfo *result) {
+	if (r2r_print_lock) {
+		r_th_lock_enter (r2r_print_lock);
+	}
 	if (result->run_failed) {
 		printf (Color_RED "RUN FAILED (e.g. wrong radare2 path)" Color_RESET "\n");
+		if (r2r_print_lock) { r_th_lock_leave (r2r_print_lock); }
 		return;
 	}
 	switch (result->test->type) {
-	case R2R_TEST_TYPE_CMD: {
-		r2r_run_cmd_test (config, result->test->cmd_test, print_runner, NULL);
-		const char *expect = result->test->cmd_test->expect.value;
-		const char *out = result->proc_out? result->proc_out->out: "";
-		const char *regexp_out = result->test->cmd_test->regexp_out.value;
-		if ((expect || regexp_out) && !r_test_cmp_cmd_output (out, expect, regexp_out)) {
-			printf ("-- stdout\n");
-			print_diff (out, expect, false, regexp_out);
+	case R2R_TEST_TYPE_CMD:
+		{
+			r2r_run_cmd_test (config, result->test->cmd_test, print_runner, NULL);
+			const char *expect = result->test->cmd_test->expect.value;
+			const char *out = result->proc_out? result->proc_out->out: "";
+			const char *regexp_out = result->test->cmd_test->regexp_out.value;
+			if ((expect || regexp_out) && !r_test_cmp_cmd_output (out, expect, regexp_out)) {
+				printf ("-- stdout\n");
+				print_diff (out, expect, false, regexp_out);
+			}
+			expect = result->test->cmd_test->expect_err.value;
+			const char *err = result->proc_out? result->proc_out->err: "";
+			const char *regexp_err = result->test->cmd_test->regexp_err.value;
+			if ((expect || regexp_err) && !r_test_cmp_cmd_output (err, expect, regexp_err)) {
+				printf ("-- stderr\n");
+				print_diff (err, expect, false, regexp_err);
+			} else if (*err) {
+				printf ("-- stderr\n%s\n", err);
+			}
+			if (result->proc_out && result->proc_out->ret != 0) {
+				printf ("-- exit status: "Color_RED"%d"Color_RESET"\n", result->proc_out->ret);
+			}
+			break;
 		}
-		expect = result->test->cmd_test->expect_err.value;
-		const char *err = result->proc_out? result->proc_out->err: "";
-		const char *regexp_err = result->test->cmd_test->regexp_err.value;
-		if ((expect || regexp_err) && !r_test_cmp_cmd_output (err, expect, regexp_err)) {
-			printf ("-- stderr\n");
-			print_diff (err, expect, false, regexp_err);
-		} else if (*err) {
-			printf ("-- stderr\n%s\n", err);
-		}
-		if (result->proc_out && result->proc_out->ret != 0) {
-			printf ("-- exit status: "Color_RED"%d"Color_RESET"\n", result->proc_out->ret);
-		}
-		break;
-	}
 	case R2R_TEST_TYPE_ASM:
 		// TODO
 		break;
@@ -1014,6 +1030,9 @@ static void print_result_diff(R2RRunConfig *config, R2RTestResultInfo *result) {
 		printf ("-- exit status: "Color_RED"%d"Color_RESET"\n", result->proc_out->ret);
 		break;
 	}
+	if (r2r_print_lock) {
+		r_th_lock_leave (r2r_print_lock);
+	}
 }
 
 static void print_new_results(R2RState *state, ut64 prev_completed) {
