@@ -1,7 +1,8 @@
-/* radare - LGPL - Copyright 2019-2024 - pancake, thestr4ng3r */
+/* radare - LGPL - Copyright 2019-2025 - pancake, thestr4ng3r */
 
 #include <r_anal.h>
 #include <r_hash.h>
+#include <r_core.h>
 
 #define unwrap(rbnode) container_of (rbnode, RAnalBlock, _rb)
 
@@ -62,9 +63,6 @@ R_API void r_anal_block_ref(RAnalBlock *bb) {
 
 static RAnalBlock *block_new(RAnal *a, ut64 addr, ut64 size) {
 	RAnalBlock *block = R_NEW0 (RAnalBlock);
-	if (!block) {
-		return NULL;
-	}
 	block->addr = addr;
 	block->size = size;
 	block->anal = a;
@@ -164,6 +162,7 @@ static bool block_list_cb(RAnalBlock *block, void *user) {
 }
 
 R_API RList *r_anal_get_blocks_in(RAnal *anal, ut64 addr) {
+	R_RETURN_VAL_IF_FAIL (anal, NULL);
 	RList *list = r_list_newf ((RListFree)r_anal_block_unref);
 	if (list) {
 		r_anal_blocks_foreach_in (anal, addr, block_list_cb, list);
@@ -196,6 +195,7 @@ R_API void r_anal_blocks_foreach_intersect(RAnal *anal, ut64 addr, ut64 size, RA
 }
 
 R_API RList *r_anal_get_blocks_intersect(RAnal *anal, ut64 addr, ut64 size) {
+	R_RETURN_VAL_IF_FAIL (anal, NULL);
 	RList *list = r_list_newf ((RListFree)r_anal_block_unref);
 	if (R_LIKELY (list)) {
 		r_anal_blocks_foreach_intersect (anal, addr, size, block_list_cb, list);
@@ -204,6 +204,7 @@ R_API RList *r_anal_get_blocks_intersect(RAnal *anal, ut64 addr, ut64 size) {
 }
 
 R_API RAnalBlock *r_anal_create_block(RAnal *anal, ut64 addr, ut64 size) {
+	R_RETURN_VAL_IF_FAIL (anal, NULL);
 	if (r_anal_get_block_at (anal, addr)) {
 		return NULL;
 	}
@@ -256,6 +257,7 @@ R_API void r_anal_block_set_size(RAnalBlock *block, ut64 size) {
 }
 
 R_API bool r_anal_block_relocate(RAnalBlock *block, ut64 addr, ut64 size) {
+	R_RETURN_VAL_IF_FAIL (block, false);
 	if (block->addr == addr) {
 		r_anal_block_set_size (block, size);
 		r_anal_block_update_hash (block);
@@ -298,6 +300,7 @@ R_API bool r_anal_block_relocate(RAnalBlock *block, ut64 addr, ut64 size) {
 }
 
 R_API RAnalBlock *r_anal_block_split(RAnalBlock *bbi, ut64 addr) {
+	R_RETURN_VAL_IF_FAIL (bbi, NULL);
 	RAnal *anal = bbi->anal;
 	R_RETURN_VAL_IF_FAIL (bbi && addr >= bbi->addr && addr < bbi->addr + bbi->size && addr != UT64_MAX, 0);
 	if (addr == bbi->addr) {
@@ -363,6 +366,7 @@ R_API RAnalBlock *r_anal_block_split(RAnalBlock *bbi, ut64 addr) {
 }
 
 R_API bool r_anal_block_merge(RAnalBlock *a, RAnalBlock *b) {
+	R_RETURN_VAL_IF_FAIL (a && b, false);
 	if (!r_anal_block_is_contiguous (a, b)) {
 		return false;
 	}
@@ -523,7 +527,9 @@ R_API bool r_anal_block_recurse_followthrough(RAnalBlock *block, RAnalBlockCb cb
 	ht_up_insert (ctx.visited, block->addr, NULL);
 	r_pvector_push (&ctx.to_visit, block);
 
-	while (!r_pvector_empty (&ctx.to_visit) && !r_cons_is_breaked ()) {
+	RCore *core = ctx.anal->coreb.core;
+	RCons *cons = core->cons;
+	while (!r_pvector_empty (&ctx.to_visit) && !r_cons_is_breaked (cons)) {
 		RAnalBlock *cur = r_pvector_pop (&ctx.to_visit);
 		if (cb (cur, user)) {
 			r_anal_block_successor_addrs_foreach (cur, block_recurse_successor_cb, &ctx);
@@ -563,6 +569,9 @@ R_API bool r_anal_block_recurse_depth_first(RAnalBlock *block, RAnalBlockCb cb,
 		goto beach;
 	}
 	do {
+		if (path.len < 1) {
+			break;
+		}
 		RecurseDepthFirstCtx *cur_ctx = r_vector_index_ptr (&path, path.len - 1);
 		cur_bb = cur_ctx->bb;
 		if (cur_bb->jump != UT64_MAX && !ht_up_find_kv (visited, cur_bb->jump, NULL)) {
@@ -619,6 +628,7 @@ static bool recurse_list_cb(RAnalBlock *block, void *user) {
 }
 
 R_API RList *r_anal_block_recurse_list(RAnalBlock *block) {
+	R_RETURN_VAL_IF_FAIL (block, NULL);
 	RList *ret = r_list_newf ((RListFree)r_anal_block_unref);
 	if (ret) {
 		r_anal_block_recurse (block, recurse_list_cb, ret);
@@ -634,6 +644,7 @@ R_API void r_anal_block_add_switch_case(RAnalBlock *block, ut64 switch_addr, ut6
 }
 
 R_API bool r_anal_block_op_starts_at(RAnalBlock *bb, ut64 addr) {
+	R_RETURN_VAL_IF_FAIL (bb, false);
 	if (!r_anal_block_contains (bb, addr)) {
 		return false;
 	}
@@ -692,7 +703,7 @@ static ut64 bb_addr_for(RAnal *a, ut64 n) {
 	return n;
 }
 
-R_API R_NULLABLE RList/*<RAnalBlock *>*/ *r_anal_block_shortest_path(RAnalBlock *block, ut64 dst) {
+R_API RList/*<RAnalBlock *>*/ * R_NULLABLE r_anal_block_shortest_path(RAnalBlock *block, ut64 dst) {
 	ut64 dstbb_addr = bb_addr_for (block->anal, dst);
 
 	RList *ret = NULL;
@@ -809,9 +820,6 @@ static bool noreturn_successors_cb(RAnalBlock *block, void *user) {
 	}
 	HtUP *succs = user;
 	NoreturnSuccessor *succ = R_NEW0 (NoreturnSuccessor);
-	if (!succ) {
-		return false;
-	}
 	r_anal_block_ref (block);
 	succ->block = block;
 	succ->reachable = false; // reset for first iteration
@@ -892,11 +900,11 @@ R_API RAnalBlock *r_anal_block_chop_noreturn(RAnalBlock *block, ut64 addr) {
 
 	// Free/unref BEFORE doing the merge!
 	// Some of the blocks might not be valid anymore later!
-	r_anal_block_unref (block);
 	ht_up_free (succs);
 
 	ut64 block_addr = block->addr; // save the addr to identify the block. the automerge might free it so we must not use the pointer!
 
+	r_anal_block_unref (block);
 	// Do the actual merge
 	r_anal_block_automerge (&merge_blocks);
 
