@@ -1,8 +1,6 @@
-/* radare - LGPL - Copyright 2009-2020 - pancake, nibble */
+/* radare - LGPL - Copyright 2009-2025 - pancake, nibble */
 
 #include <r_core.h>
-#include <r_util.h>
-#include <r_cons.h>
 
 #define mid_down_refline(a, r) ((r)->from > (r)->to && (a) < (r)->from && (a) > (r)->to)
 #define mid_up_refline(a, r) ((r)->from < (r)->to && (a) > (r)->from && (a) < (r)->to)
@@ -23,49 +21,45 @@ static int cmp_by_ref_lvl(const RAnalRefline *a, const RAnalRefline *b) {
 	return (a->level < b->level) - (a->level > b->level);
 }
 
-static ReflineEnd *refline_end_new(ut64 val, bool is_from, RAnalRefline *ref) {
+static ReflineEnd * R_NONNULL refline_end_new(ut64 val, bool is_from, RAnalRefline *ref) {
 	ReflineEnd *re = R_NEW0 (struct refline_end);
-	if (!re) {
-		return NULL;
-	}
 	re->val = val;
 	re->is_from = is_from;
 	re->r = ref;
 	return re;
 }
 
-static bool add_refline(RList *list, RList *sten, ut64 addr, ut64 to, int *idx) {
-	ReflineEnd *re1, *re2;
-	RAnalRefline *item = R_NEW0 (RAnalRefline);
-	if (!item) {
-		return false;
+static bool add_refline(RList *list, RList *sten, ut64 addr, ut64 to, int *idx, int type, int splitmode) {
+	if (splitmode) {
+		if (splitmode > 0) {
+			if (addr > to) {
+				return true;
+			}
+		} else {
+			if (addr < to) {
+				return true;
+			}
+		}
 	}
+	RAnalRefline *item = R_NEW0 (RAnalRefline);
 	item->from = addr;
 	item->to = to;
 	item->index = *idx;
 	item->level = -1;
+	item->type = type;
 	item->direction = (to > addr)? 1: -1;
 	*idx += 1;
 	r_list_append (list, item);
 
-	re1 = refline_end_new (item->from, true, item);
-	if (!re1) {
-		free (item);
-		return false;
-	}
+	ReflineEnd *re1 = refline_end_new (item->from, true, item);
 	r_list_add_sorted (sten, re1, (RListComparator)cmp_asc);
 
-	re2 = refline_end_new (item->to, false, item);
-	if (!re2) {
-		free (re1);
-		free (item);
-		return false;
-	}
+	ReflineEnd *re2 = refline_end_new (item->to, false, item);
 	r_list_add_sorted (sten, re2, (RListComparator)cmp_asc);
 	return true;
 }
 
-R_API void r_anal_reflines_free (RAnalRefline *rl) {
+R_API void r_anal_reflines_free(RAnalRefline *rl) {
 	free (rl);
 }
 
@@ -76,19 +70,20 @@ R_API void r_anal_reflines_free (RAnalRefline *rl) {
  *
  * nlines - max number of lines of code to consider
  * linesout - true if you want to display lines that go outside of the scope [addr;addr+len)
- * linescall - true if you want to display call lines */
-R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall) {
-	RList *list, *sten;
+ * linescall - true if you want to display call lines
+ * splitmode - 0, -1, 1 */
+R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 len, int nlines, int linesout, int linescall, int splitmode) {
 	RListIter *iter;
-	RAnalOp op;
+	RAnalOp op = {0};
 	struct refline_end *el;
 	const ut8 *ptr = buf;
 	const ut8 *end = buf + len;
 	ut8 *free_levels;
 	int sz = 0, count = 0;
 	ut64 opc = addr;
+	RCore *core = anal->coreb.core;
+	RCons *cons = core->cons;
 
-	memset (&op, 0, sizeof (op));
 	/*
 	 * 1) find all reflines
 	 * 2) sort "from"s and "to"s in a list
@@ -100,17 +95,14 @@ R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 le
 	 *        refline, we free that level.
 	 */
 
-	list = r_list_newf (free);
+	RList *list = r_list_newf (free);
 	if (!list) {
 		return NULL;
 	}
-	sten = r_list_newf ((RListFree)free);
-	if (!sten) {
-		goto list_err;
-	}
-	r_cons_break_push (NULL, NULL);
+	RList *sten = r_list_newf ((RListFree)free);
+	r_cons_break_push (cons, NULL, NULL);
 	/* analyze code block */
-	while (ptr < end && !r_cons_is_breaked ()) {
+	while (ptr < end && !r_cons_is_breaked (cons)) {
 		if (nlines != -1) {
 			if (!nlines) {
 				break;
@@ -120,28 +112,33 @@ R_API RList *r_anal_reflines_get(RAnal *anal, ut64 addr, const ut8 *buf, ut64 le
 		if (anal->maxreflines && count > anal->maxreflines) {
 			break;
 		}
+		ut64 skip = 0;
+		ut64 bind_addr = 0;
 		addr += sz;
 		{
 			RPVector *metas = r_meta_get_all_at (anal, addr);
 			if (metas) {
 				void **it;
-				ut64 skip = 0;
 				r_pvector_foreach (metas, it) {
 					RIntervalNode *node = *it;
 					RAnalMetaItem *meta = node->data;
 					switch (meta->type) {
+					case R_META_TYPE_BIND:
+						bind_addr = r_num_math (NULL, meta->str);
+						break;
 					case R_META_TYPE_DATA:
 					case R_META_TYPE_STRING:
 					case R_META_TYPE_HIDE:
 					case R_META_TYPE_FORMAT:
 					case R_META_TYPE_MAGIC:
 						skip = r_meta_node_size (node);
-						goto do_skip;
+						// goto doskip;
+						break;
 					default:
 						break;
 					}
 				}
-do_skip:
+// doskip:
 				r_pvector_free (metas);
 				if (skip) {
 					ptr += skip;
@@ -150,6 +147,10 @@ do_skip:
 				}
 			}
 		}
+		if (bind_addr != 0 && bind_addr != UT64_MAX) {
+			add_refline (list, sten, addr, bind_addr, &count, 'b', splitmode);
+			bind_addr = UT64_MAX;
+		}
 		if (!anal->iob.is_valid_offset (anal->iob.io, addr, 1)) {
 			const int size = 4;
 			ptr += size;
@@ -159,7 +160,7 @@ do_skip:
 
 		// This can segfault if opcode length and buffer check fails
 		r_anal_op_fini (&op);
-		int rc = r_anal_op (anal, &op, addr, ptr, (int)(end - ptr), R_ANAL_OP_MASK_BASIC | R_ANAL_OP_MASK_HINT);
+		int rc = r_anal_op (anal, &op, addr, ptr, (int)(end - ptr), R_ARCH_OP_MASK_BASIC | R_ARCH_OP_MASK_HINT);
 		if (rc <= 0) {
 			sz = 1;
 			goto __next;
@@ -178,16 +179,19 @@ do_skip:
 			}
 		case R_ANAL_OP_TYPE_CJMP:
 		case R_ANAL_OP_TYPE_JMP:
+			if (op.jump == UT64_MAX) {
+				break;
+			}
 			if ((!linesout && (op.jump > opc + len || op.jump < opc)) || !op.jump) {
 				break;
 			}
-			if (!add_refline (list, sten, addr, op.jump, &count)) {
+			if (!add_refline (list, sten, addr, op.jump, &count, 'j', splitmode)) {
 				r_anal_op_fini (&op);
 				goto sten_err;
 			}
 			// add false branch in case its set and its not a call, useful for bf, maybe others
 			if (!op.delay && op.fail != UT64_MAX && op.fail != addr + op.size) {
-				if (!add_refline (list, sten, addr, op.fail, &count)) {
+				if (!add_refline (list, sten, addr, op.fail, &count, 'f', splitmode)) {
 					r_anal_op_fini (&op);
 					goto sten_err;
 				}
@@ -206,7 +210,7 @@ do_skip:
 				if (!linesout && (op.jump > opc + len || op.jump < opc)) {
 					goto __next;
 				}
-				if (!add_refline (list, sten, op.switch_op->addr, caseop->jump, &count)) {
+				if (!add_refline (list, sten, op.switch_op->addr, caseop->jump, &count, 'j', splitmode)) {
 					r_anal_op_fini (&op);
 					goto sten_err;
 				}
@@ -218,12 +222,9 @@ do_skip:
 		ptr += sz;
 	}
 	r_anal_op_fini (&op);
-	r_cons_break_pop ();
+	r_cons_break_pop (cons);
 
 	free_levels = R_NEWS0 (ut8, r_list_length (list) + 1);
-	if (!free_levels) {
-		goto sten_err;
-	}
 	int min = 0;
 
 	r_list_foreach (sten, iter, el) {
@@ -259,7 +260,7 @@ do_skip:
 	return list;
 
 sten_err:
-list_err:
+	r_anal_op_fini (&op);
 	r_list_free (sten);
 	r_list_free (list);
 	return NULL;
@@ -278,19 +279,23 @@ R_API int r_anal_reflines_middle(RAnal *a, RList* /*<RAnalRefline>*/ list, ut64
 	return false;
 }
 
+static inline const char* colchar(RAnalRefline *ref) {
+	return (ref->type == 'b') ? "!" : "|";
+}
+
 static const char* get_corner_char(RAnalRefline *ref, ut64 addr, bool is_middle_before) {
 	if (ref->from == ref->to) {
 		return "@";
 	}
 	if (addr == ref->to) {
 		if (is_middle_before) {
-			return (ref->from > ref->to) ? " " : "|";
+			return (ref->from > ref->to) ? " " : colchar (ref);
 		}
 		return (ref->from > ref->to) ? "." : "`";
 	}
 	if (addr == ref->from) {
 		if (is_middle_before) {
-			return (ref->from > ref->to) ? "|" : " ";
+			return (ref->from > ref->to) ? colchar (ref) : " ";
 		}
 		return (ref->from > ref->to) ? "`" : ",";
 	}
@@ -345,10 +350,9 @@ static inline bool refline_kept(RAnalRefline *ref, bool middle_after, ut64 addr)
 // TODO: this is TOO SLOW. do not iterate over all reflines or gtfo
 R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 	RCore *core = _core;
-	RCons *cons = core->cons;
+	R_RETURN_VAL_IF_FAIL (core && core->anal, NULL);
+	RConsContext *ctx= core->cons->context;
 	RAnal *anal = core->anal;
-	RBuffer *b;
-	RBuffer *c;
 	RListIter *iter;
 	RAnalRefline *ref;
 	int l;
@@ -356,17 +360,22 @@ R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 	int dir = 0, pos = -1, max_level = -1;
 	bool middle_before = opts & R_ANAL_REFLINE_TYPE_MIDDLE_BEFORE;
 	bool middle_after = opts & R_ANAL_REFLINE_TYPE_MIDDLE_AFTER;
+	bool split_mode = opts & R_ANAL_REFLINE_TYPE_SPLIT;
 	char *str = NULL;
 	char *col_str = NULL;
-
-	r_return_val_if_fail (cons && anal && anal->reflines, NULL);
+#if 0
+	if (!anal->reflines) {
+		return NULL;
+	}
+#endif
+	RList *reflines = split_mode ? anal->reflines2: anal->reflines;
 
 	RList *lvls = r_list_new ();
 	if (!lvls) {
 		return NULL;
 	}
-	r_list_foreach (anal->reflines, iter, ref) {
-		if (core->cons && core->cons->context->breaked) {
+	r_list_foreach (reflines, iter, ref) {
+		if (ctx->breaked) {
 			r_list_free (lvls);
 			return NULL;
 		}
@@ -374,12 +383,12 @@ R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 			r_list_add_sorted (lvls, (void *)ref, (RListComparator)cmp_by_ref_lvl);
 		}
 	}
-	b = r_buf_new ();
-	c = r_buf_new ();
+	RBuffer *b = r_buf_new ();
+	RBuffer *c = r_buf_new ();
 	r_buf_append_string (c, " ");
 	r_buf_append_string (b, " ");
 	r_list_foreach (lvls, iter, ref) {
-		if (core->cons && core->cons->context->breaked) {
+		if (ctx->breaked) {
 			r_list_free (lvls);
 			r_buf_free (b);
 			r_buf_free (c);
@@ -423,7 +432,7 @@ R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 				r_buf_append_string (b, ":");
 				r_buf_append_string (c, "t");
 			} else {
-				r_buf_append_string (b, "|");
+				r_buf_append_string (b, colchar (ref));
 				r_buf_append_string (c, "d");
 			}
 			pos = ref->level;
@@ -434,15 +443,15 @@ R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 	}
 	add_spaces (c, 0, pos, wide);
 	add_spaces (b, 0, pos, wide);
-	str = r_buf_to_string (b);
-	col_str = r_buf_to_string (c);
+	str = r_buf_tostring (b);
+	col_str = r_buf_tostring (c);
 	r_buf_free (b);
 	r_buf_free (c);
 	b = NULL;
 	c = NULL;
 	if (!str || !col_str) {
 		r_list_free (lvls);
-		//r_buf_free_to_string already free b and if that is the case
+		//r_buf_free_tostring already free b and if that is the case
 		//b will be NULL and r_buf_free will return but if there was
 		//an error we free b here so in other words is safe
 		r_buf_free (b);
@@ -483,7 +492,9 @@ R_API RAnalRefStr *r_anal_reflines_str(void *_core, ut64 addr, int opts) {
 }
 
 R_API void r_anal_reflines_str_free(RAnalRefStr *refstr) {
-	free (refstr->str);
-	free (refstr->cols);
-	free (refstr);
+	if (refstr) {
+		free (refstr->str);
+		free (refstr->cols);
+		free (refstr);
+	}
 }
