@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2021-2023 - junchao82@qq.com;zhaojunchao@loongson.cn love lanhy*/
+/* radare - LGPL - Copyright 2021-2024 - junchao82@qq.com;zhaojunchao@loongson.cn love lanhy*/
 
 #include <r_arch.h>
 #include "../../include/disas-asm.h"
@@ -1197,7 +1197,7 @@ static void insn_memory_error_func(int status, bfd_vma memaddr, struct disassemb
 	//TODO
 }
 
-static bool decode(RArchSession *as, RAnalOp *op, RAnalOpMask mask) {
+static bool decode(RArchSession *as, RAnalOp *op, RArchDecodeMask mask) {
 	const ut64 addr = op->addr;
 	const ut8 *b = op->bytes;
 	const int len = op->size;
@@ -1408,8 +1408,9 @@ static bool fini(RArchSession *s) {
 const RArchPlugin r_arch_plugin_loongarch_gnu = {
 	.meta = {
 		.name = "loongarch",
-		.desc = "loongson loongarch code analysis plugin",
-		.license = "LGPL3",
+		.author = "junchao82,zhaojunchao",
+		.desc = "Loongson / loongarch / mips-like architecture",
+		.license = "LGPL-3.0-only",
 	},
 	.arch = "loongarch",
 	.bits = 64,
