@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2018-2024 - pancake, mrmacete, keegan */
+/* radare2 - LGPL - Copyright 2018-2025 - pancake, mrmacete, keegan */
 
 #include <r_core.h>
 #include <sdb/ht_su.h>
@@ -21,6 +21,7 @@ typedef struct _r_dyldcache {
 	ut64 *hdr_offset;
 	ut64 *hdr_overhead;
 	ut32 *maps_index;
+	ut64 *maps_flags;
 	ut32 n_hdr;
 	cache_map_t *maps;
 	ut32 n_maps;
@@ -33,6 +34,7 @@ typedef struct _r_dyldcache {
 	RDyldLocSym *locsym;
 	objc_cache_opt_info *oi;
 	bool objc_opt_info_loaded;
+	bool images_are_global;
 } RDyldCache;
 
 typedef struct _r_bin_image {
@@ -90,7 +92,7 @@ static cache_img_t *read_cache_images(RBuffer *cache_buf, cache_hdr_t *hdr, ut64
 		return NULL;
 	}
 
-	if (r_buf_fread_at (cache_buf, hdr->imagesOffset, (ut8*) images, "3l2i", hdr->imagesCount) != size) {
+	if (r_buf_fread_at (cache_buf, (ut64) hdr->imagesOffset + hdr_offset, (ut8*) images, "3l2i", hdr->imagesCount) != size) {
 		R_FREE (images);
 		return NULL;
 	}
@@ -171,9 +173,9 @@ static RDyldLocSym *r_dyld_locsym_new(RDyldCache *cache) {
 			if (!regular_entries) {
 				goto beach;
 			}
-			if (r_buf_fread_at (cache->buf, hdr->localSymbolsOffset + info->entriesOffset, (ut8*) regular_entries, "iii",
-					info->entriesCount) != entries_size) {
-				R_LOG_ERROR ("incomplete local symbol entries");
+			if (r_buf_fread_at (cache->buf, hdr->localSymbolsOffset + info->entriesOffset,
+					(ut8*) regular_entries, "iii", info->entriesCount) != entries_size) {
+				R_LOG_ERROR ("Incomplete local symbol entries");
 				goto beach;
 			}
 			entries = regular_entries;
@@ -281,6 +283,7 @@ static void r_dyldcache_free(RDyldCache *cache) {
 	R_FREE (cache->hdr);
 	R_FREE (cache->maps);
 	R_FREE (cache->maps_index);
+	R_FREE (cache->maps_flags);
 	R_FREE (cache->hdr_offset);
 	R_FREE (cache->hdr_overhead);
 	R_FREE (cache->accel);
@@ -329,7 +332,7 @@ static struct MACH0_(obj_t) *bin_to_mach0(RBinFile *bf, RDyldBinImage *bin) {
 		opts.symbols_off = bin->symbols_off - bin->hdr_offset;
 	}
 
-	struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (buf, &opts);
+	struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (bf, buf, &opts);
 	if (mach0) {
 		mach0->user = cache;
 		mach0->va2pa = &bin_obj_va2pa;
@@ -374,7 +377,8 @@ static bool check(RBinFile *bf, RBuffer *buf) {
 	return check_magic (hdr);
 }
 
-static cache_imgxtr_t *read_cache_imgextra(RBuffer *cache_buf, cache_hdr_t *hdr, cache_accel_t *accel) {
+static cache_imgxtr_t *read_cache_imgextra(RBuffer *cache_buf, cache_hdr_t *hdr, cache_accel_t *accel, ut64 * out_count) {
+	*out_count = 0;
 	if (!cache_buf || !hdr || !hdr->imagesCount || !accel || !accel->imageExtrasCount || !accel->imagesExtrasOffset) {
 		return NULL;
 	}
@@ -390,6 +394,8 @@ static cache_imgxtr_t *read_cache_imgextra(RBuffer *cache_buf, cache_hdr_t *hdr,
 		return NULL;
 	}
 
+	*out_count = accel->imageExtrasCount;
+
 	return images;
 }
 
@@ -560,6 +566,7 @@ static void create_cache_bins(RBinFile *bf, RDyldCache *cache) {
 	char *target_libs = NULL;
 	RList *target_lib_names = NULL;
 
+	ut64 extras_count = 0;
 	cache_imgxtr_t *extras = NULL;
 	if (!bins) {
 		return;
@@ -587,15 +594,25 @@ static void create_cache_bins(RBinFile *bf, RDyldCache *cache) {
 		eprintf ("bin.dyldcache: list of names to avoid loading all the files in memory.\n");
 	}
 
+	cache_img_t *img = NULL;
+	if (cache->images_are_global) {
+		img = read_cache_images (cache->buf, cache->hdr, 0);
+		if (!img) {
+			return;
+		}
+	}
+
 	ut32 i;
 	for (i = 0; i < cache->n_hdr; i++) {
 		cache_hdr_t *hdr = &cache->hdr[i];
 		ut64 hdr_offset = cache->hdr_offset[i];
 		ut64 hdr_overhead = cache->hdr_overhead[i];
 		ut32 maps_index = cache->maps_index[i];
-		cache_img_t *img = read_cache_images (cache->buf, hdr, hdr_offset);
 		if (!img) {
-			goto next;
+			img = read_cache_images (cache->buf, hdr, hdr_offset);
+			if (!img) {
+				goto next;
+			}
 		}
 
 		ut32 j;
@@ -613,7 +630,7 @@ static void create_cache_bins(RBinFile *bf, RDyldCache *cache) {
 					goto next;
 				}
 
-				extras = read_cache_imgextra (cache->buf, hdr, cache->accel);
+				extras = read_cache_imgextra (cache->buf, hdr, cache->accel, &extras_count);
 				if (!extras) {
 					goto next;
 				}
@@ -640,7 +657,7 @@ static void create_cache_bins(RBinFile *bf, RDyldCache *cache) {
 				R_FREE (lib_name);
 				deps[j]++;
 
-				if (extras && depArray) {
+				if (extras && depArray && j < extras_count) {
 					ut32 k;
 					for (k = extras[j].dependentsStartArrayIndex; k < depListCount && depArray[k] != 0xffff; k++) {
 						ut16 dep_index = depArray[k] & 0x7fff;
@@ -732,8 +749,11 @@ static void create_cache_bins(RBinFile *bf, RDyldCache *cache) {
 next:
 		R_FREE (depArray);
 		R_FREE (extras);
-		R_FREE (img);
+		if (!cache->images_are_global) {
+			R_FREE (img);
+		}
 	}
+	R_FREE (img);
 end:
 	if (r_list_empty (bins)) {
 		r_list_free (bins);
@@ -788,6 +808,7 @@ static void populate_cache_headers(RDyldCache *cache) {
 	ut64 overheads[MAX_N_HDR];
 	ut64 offset = 0, overhead = 0;
 	ut64 buf_size = r_buf_size (cache->buf);
+	ut32 n_zero_images = 0;
 	do {
 		offsets[cache->n_hdr] = offset;
 		overheads[cache->n_hdr] = overhead;
@@ -796,6 +817,10 @@ static void populate_cache_headers(RDyldCache *cache) {
 			break;
 		}
 
+		if (!h->imagesCount && !h->imagesOffset) {
+			n_zero_images++;
+		}
+
 		ut64 size = h->codeSignatureOffset + h->codeSignatureSize;
 		if (!size || size > buf_size) {
 			break;
@@ -808,12 +833,9 @@ static void populate_cache_headers(RDyldCache *cache) {
 		x += offset; \
 	}
 
-		SHIFT_MAYBE (h->mappingOffset);
-		SHIFT_MAYBE (h->imagesOffset);
 		SHIFT_MAYBE (h->codeSignatureOffset);
 		SHIFT_MAYBE (h->slideInfoOffset);
 		SHIFT_MAYBE (h->localSymbolsOffset);
-		SHIFT_MAYBE (h->branchPoolsOffset);
 		SHIFT_MAYBE (h->imagesTextOffset);
 
 		offset += size;
@@ -843,13 +865,23 @@ static void populate_cache_headers(RDyldCache *cache) {
 	memcpy (cache->hdr_offset, offsets, cache->n_hdr * sizeof (ut64));
 	memcpy (cache->hdr_overhead, overheads, cache->n_hdr * sizeof (ut64));
 
+	cache->images_are_global = n_zero_images == cache->n_hdr - 1;
+
 	ut32 i = 0;
 	RListIter *iter;
 	cache_hdr_t *item;
+	cache_hdr_t *prev_h = NULL;
 	r_list_foreach (hdrs, iter, item) {
 		if (i >= cache->n_hdr) {
 			break;
 		}
+		if (cache->images_are_global) {
+			if (!item->imagesCount && !item->imagesOffset && prev_h) {
+				item->imagesCount = prev_h->imagesCount;
+				item->imagesOffset = prev_h->imagesOffset;
+			}
+			prev_h = item;
+		}
 		memcpy (&cache->hdr[i++], item, sizeof (cache_hdr_t));
 	}
 
@@ -897,7 +929,10 @@ static void populate_cache_maps(RDyldCache *cache) {
 		cache->n_maps = 0;
 		return;
 	}
-
+	ut64 * maps_flags = NULL;
+	if (cache->hdr->mappingOffset >= 0x140) {
+		maps_flags = R_NEWS0 (ut64, n_maps);
+	}
 	ut32 next_map = 0;
 	for (i = 0; i < cache->n_hdr; i++) {
 		cache_hdr_t *hdr = &cache->hdr[i];
@@ -906,21 +941,34 @@ static void populate_cache_maps(RDyldCache *cache) {
 		if (!hdr->mappingCount || !hdr->mappingOffset) {
 			continue;
 		}
+		ut64 hdr_offset = cache->hdr_offset[i];
+		ut64 mapping_slide_offset = 0;
+		if (maps_flags && hdr->mappingOffset >= 0x140) {
+			ut32 mapping_slide_count = r_buf_read_le32_at (cache->buf, 0x13c + hdr_offset);
+			if (mapping_slide_count == hdr->mappingCount) {
+				mapping_slide_offset = r_buf_read_le32_at (cache->buf, 0x138 + hdr_offset);
+			}
+		}
+
 		ut64 size = sizeof (cache_map_t) * hdr->mappingCount;
-		if (r_buf_fread_at (cache->buf, hdr->mappingOffset, (ut8*) &maps[next_map], "3l2i", hdr->mappingCount) != size) {
+		if (r_buf_fread_at (cache->buf, (ut64) hdr->mappingOffset + hdr_offset, (ut8*) &maps[next_map], "3l2i", hdr->mappingCount) != size) {
 			continue;
 		}
 		ut32 j;
-		ut64 hdr_offset = cache->hdr_offset[i];
 		for (j = 0; j < hdr->mappingCount; j++) {
 			cache_map_t *map = &maps[next_map + j];
 			map->fileOffset += hdr_offset;
+			if (maps_flags && mapping_slide_offset) {
+				maps_flags[next_map + j] = r_buf_read_le64_at(cache->buf,
+						mapping_slide_offset + hdr_offset + (ut64)j * sizeof (cache_mapping_slide) + offsetof(cache_mapping_slide, flags));
+			}
 		}
 		next_map += hdr->mappingCount;
 	}
 
 	cache->maps = maps;
 	cache->n_maps = next_map;
+	cache->maps_flags = maps_flags;
 }
 
 static cache_accel_t *read_cache_accel(RBuffer *cache_buf, cache_hdr_t *hdr, cache_map_t *maps, int n_maps) {
@@ -967,11 +1015,11 @@ static objc_cache_opt_info *get_objc_opt_info(RBinFile *bf, RDyldCache *cache) {
 
 		struct MACH0_(opts_t) opts = {0};
 		MACH0_(opts_set_default) (&opts, bf);
-		opts.verbose = bf->rbin->verbose;
+		opts.verbose = bf->rbin->options.verbose;
 		opts.header_at = bin->header_at;
 		opts.symbols_off = 0;
 
-		struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (cache->buf, &opts);
+		struct MACH0_(obj_t) *mach0 = MACH0_(new_buf) (bf, cache->buf, &opts);
 		if (!mach0) {
 			goto beach;
 		}
@@ -1304,10 +1352,11 @@ static RList *sections(RBinFile *bf) {
 	RListIter *iter;
 	RDyldBinImage *bin;
 	ut32 i = 0;
+	RCons *cons = bf->rbin->consb.cons;
 	RConsIsBreaked is_breaked = (bf->rbin && bf->rbin->consb.is_breaked)? bf->rbin->consb.is_breaked: NULL;
 	r_list_foreach (cache->bins, iter, bin) {
 		i++;
-		if (is_breaked && is_breaked ()) {
+		if (is_breaked && is_breaked (cons)) {
 			eprintf ("Parsing sections stopped %d / %d\n", i, r_list_length (cache->bins));
 			break;
 		}
@@ -1348,13 +1397,22 @@ static RList *sections(RBinFile *bf) {
 		ut32 maps_index = cache->maps_index[i];
 		cache_map_t * first_map = &cache->maps[maps_index];
 
-		bool is_stubs = hdr->imagesCount == 0 &&
-			hdr->mappingCount == 1 &&
-			first_map->size > 0x4000 &&
-			first_map->initProt == 5;
+		if (cache->maps_flags) {
+			if ((cache->maps_flags[maps_index] & DYLD_CACHE_MAPPING_TEXT_STUBS) == 0) {
+				continue;
+			}
+		} else {
+			bool is_stubs_fallback =
+				!cache->images_are_global && (
+					hdr->imagesCount == 0 &&
+					hdr->mappingCount == 1 &&
+					first_map->size > 0x4000 &&
+					first_map->initProt == 5
+				);
 
-		if (!is_stubs) {
-			continue;
+			if (!is_stubs_fallback) {
+				continue;
+			}
 		}
 
 		if (!(ptr = R_NEW0 (RBinSection))) {
@@ -1387,10 +1445,11 @@ static bool symbols_vec(RBinFile *bf) {
 	RListIter *iter;
 	RDyldBinImage *bin;
 	ut32 i = 0;
-	RConsIsBreaked is_breaked = (bf->rbin && bf->rbin->consb.is_breaked)? bf->rbin->consb.is_breaked: NULL;
+	RConsBind *consb = &bf->rbin->consb;
+	RConsIsBreaked is_breaked = (consb->is_breaked)? consb->is_breaked: NULL;
 	r_list_foreach (cache->bins, iter, bin) {
 		i++;
-		if (is_breaked && is_breaked ()) {
+		if (is_breaked && is_breaked (consb->cons)) {
 			eprintf ("Parsing symbols stopped %d / %d\n", i, r_list_length (cache->bins));
 			break;
 		}
@@ -1409,7 +1468,7 @@ static bool symbols_vec(RBinFile *bf) {
 	i = 0;
 	r_list_foreach (cache->bins, iter, bin) {
 		i++;
-		if (is_breaked && is_breaked ()) {
+		if (is_breaked && is_breaked (consb->cons)) {
 			eprintf ("Parsing symbols stopped %d / %d\n", i, r_list_length (cache->bins));
 			break;
 		}
@@ -1458,10 +1517,11 @@ static RList *classes(RBinFile *bf) {
 	RBuffer *orig_buf = bf->buf;
 	ut32 num_of_unnamed_class = 0;
 	ut32 i = 0;
+	RCons *cons = bf->rbin->consb.cons;
 	RConsIsBreaked is_breaked = (bf->rbin && bf->rbin->consb.is_breaked)? bf->rbin->consb.is_breaked: NULL;
 	r_list_foreach (cache->bins, iter, bin) {
 		i++;
-		if (is_breaked && is_breaked ()) {
+		if (is_breaked && is_breaked (cons)) {
 			eprintf ("Parsing classes stopped %d / %d\n", i, r_list_length (cache->bins));
 			break;
 		}
@@ -1483,7 +1543,7 @@ static RList *classes(RBinFile *bf) {
 			}
 
 			bool is_classlist = strstr (section->name, "__objc_classlist");
-			bool is_catlist = strstr (section->name, "__objc_catlist");
+			bool is_catlist = r_str_endswith (section->name, "__objc_catlist");
 
 			if (!is_classlist && !is_catlist) {
 				continue;
@@ -1526,7 +1586,7 @@ static RList *classes(RBinFile *bf) {
 				bf->buf = orig_buf;
 
 				if (!klass->name) {
-					if (bf->rbin->verbose) {
+					if (bf->rbin->options.verbose) {
 						R_LOG_ERROR ("KLASS failed at 0x%"PFMT64x" [pa 0x%"PFMT64x" va 0x%"PFMT64x"], is_classlist %d",
 								pointer_to_class, cursor - pointers + offset, cursor - pointers + section->vaddr,  is_classlist);
 					}
@@ -1668,8 +1728,9 @@ beach:
 RBinPlugin r_bin_plugin_dyldcache = {
 	.meta = {
 		.name = "dyldcache",
-		.desc = "dyldcache bin plugin",
-		.license = "LGPL3",
+		.desc = "Apple dynamic system library shared cache",
+		.author = "mrmacete",
+		.license = "LGPL-3.0-only",
 	},
 	.load = &load,
 	.entries = &entries,
