@@ -1,4 +1,4 @@
-/* radare2 - LGPL - Copyright 2009-2024 - pancake */
+/* radare2 - LGPL - Copyright 2009-2025 - pancake */
 
 #if R_INCLUDE_BEGIN
 
@@ -6,10 +6,14 @@
 
 char *getcommapath(RCore *core);
 
-static R_TH_LOCAL ut64 filter_offset = UT64_MAX;
-static R_TH_LOCAL int filter_format = 0;
-static R_TH_LOCAL size_t filter_count = 0;
-static R_TH_LOCAL Sdb *fscache = NULL;
+typedef struct {
+	RCore *core;
+	ut64 filter_offset;
+	int filter_format;
+	size_t filter_count;
+	PJ *pj;
+	Sdb *fscache;
+} FilterStruct;
 
 static RCoreHelpMessage help_msg_C = {
 	"Usage:", "C[-LCvsdfm*?][*?] [...]", " # Metadata management",
@@ -156,76 +160,51 @@ static RCoreHelpMessage help_msg_Cvs = {
 	NULL
 };
 
-static int remove_meta_offset(RCore *core, ut64 offset) {
-	char aoffset[SDB_NUM_BUFSZ];
-	char *aoffsetptr = sdb_itoa (offset, 16, aoffset, sizeof (aoffset));
-	if (!aoffsetptr) {
-		R_LOG_ERROR ("Failed to convert %"PFMT64x" to a key", offset);
-		return -1;
-	}
-	return sdb_unset (core->bin->cur->sdb_addrinfo, aoffsetptr, 0);
-}
-
 static bool print_meta_offset(RCore *core, ut64 addr, PJ *pj) {
-	int line, line_old, i;
-	char file[1024];
-	int colu = 0; /// addr2line function cant retrieve column info
-	int ret = r_bin_addr2line (core->bin, addr, file, sizeof (file) - 1, &line, &colu);
-	if (ret) {
-		if (pj) {
-			pj_o (pj);
-			pj_ks (pj, "file", file);
-			pj_kn (pj, "line", line);
-			pj_kn (pj, "colu", colu);
-			pj_kn (pj, "addr", addr);
-			if (r_file_exists (file)) {
-				char *row = r_file_slurp_line (file, line, 0);
-				pj_ks (pj, "text", file);
-				free (row);
-			} else {
-				// R_LOG_ERROR ("Cannot open '%s'", file);
-			}
-			pj_end (pj);
-			return ret;
-		}
-
-		r_cons_printf ("file: %s\nline: %d\ncolu: %d\naddr: 0x%08"PFMT64x"\n", file, line, colu, addr);
-		line_old = line;
-		if (line >= 2) {
-			line -= 2;
-		}
-		if (r_file_exists (file)) {
-			for (i = 0; i < 5; i++) {
-				char *row = r_file_slurp_line (file, line + i, 0);
-				if (row) {
-					r_cons_printf ("%c %.3x  %s\n", line+i == line_old ? '>' : ' ', line+i, row);
-					free (row);
-				}
-			}
+	RBinAddrline *al = r_bin_addrline_get (core->bin, addr);
+	if (!al) {
+		return false;
+	}
+	if (pj) {
+		pj_o (pj);
+		pj_ks (pj, "file", al->file);
+		pj_kn (pj, "line", al->line);
+		pj_kn (pj, "colu", al->column);
+		pj_kn (pj, "addr", addr);
+		if (r_file_exists (al->file)) {
+			char *row = r_file_slurp_line (al->file, al->line, 0);
+			pj_ks (pj, "text", row);
+			free (row);
 		} else {
-			R_LOG_ERROR ("Cannot open '%s'", file);
+			// R_LOG_ERROR ("Cannot open '%s'", file);
 		}
+		pj_end (pj);
+		return true;
 	}
-	return ret;
-}
+	int line = al->line;
 
-#if 0
-static int remove_meta_fileline(RCore *core, const char *file_line) {
-	return sdb_unset (core->bin->cur->sdb_addrinfo, file_line, 0);
-}
-
-static int print_meta_fileline(RCore *core, const char *file_line) {
-	char *meta_info = sdb_get (core->bin->cur->sdb_addrinfo, file_line, 0);
-	if (meta_info) {
-		r_cons_printf ("Meta info %s\n", meta_info);
+	r_cons_printf (core->cons, "file: %s\nline: %d\ncolu: %d\naddr: 0x%08"PFMT64x"\n", al->file, line, al->column, addr);
+	int line_old = line;
+	if (line >= 2) {
+		line -= 2;
+	}
+	if (r_file_exists (al->file)) {
+		int i;
+		for (i = 0; i < 5; i++) {
+			char *row = r_file_slurp_line (al->file, line + i, 0);
+			if (row) {
+				r_cons_printf (core->cons, "%c %.3x  %s\n", al->line + i == line_old ? '>' : ' ', line + i, row);
+				free (row);
+			}
+		}
 	} else {
-		r_cons_printf ("No meta info for %s found\n", file_line);
+		R_LOG_ERROR ("Cannot open '%s'", al->file);
 	}
-	return 0;
+	return true;
 }
-#endif
 
 static bool print_addrinfo_json(void *user, const char *k, const char *v) {
+	FilterStruct *fs = (FilterStruct *)user;
 	ut64 offset = sdb_atoi (k);
 	if (!offset || offset == UT64_MAX) {
 		return true;
@@ -236,36 +215,36 @@ static bool print_addrinfo_json(void *user, const char *k, const char *v) {
 		colonpos = strchr (subst, ':');
 	}
 	if (!colonpos) {
-		r_cons_printf ("%s\n", subst);
+		r_cons_printf (fs->core->cons, "%s\n", subst);
 	}
-	if (colonpos && (filter_offset == UT64_MAX || filter_offset == offset)) {
-		if (filter_format) {
+	if (colonpos && (fs->filter_offset == UT64_MAX || fs->filter_offset == offset)) {
+		if (fs->filter_format) {
 			*colonpos = ':';
-	//		r_cons_printf ("CL %s %s\n", k, subst);
+	//		r_cons_printf (core->cons, "CL %s %s\n", k, subst);
 		} else {
 			*colonpos = 0;
-	//		r_cons_printf ("file: %s\nline: %s\naddr: 0x%08"PFMT64x"\n", subst, colonpos + 1, offset);
+	//		r_cons_printf (core->cons, "file: %s\nline: %s\naddr: 0x%08"PFMT64x"\n", subst, colonpos + 1, offset);
 		}
-		filter_count++;
+		fs->filter_count++;
 	}
 	const char *file = subst;
 	int line = atoi (colonpos + 1);
 	ut64 addr = offset;
-	PJ *pj = (PJ*)user;
+	PJ *pj = fs->pj;
 	if (pj) {
 		pj_o (pj);
 		pj_ks (pj, "file", file);
 		pj_kn (pj, "line", line);
 		pj_kn (pj, "addr", addr);
-		const char *cached_existance = sdb_const_get (fscache, file, NULL);
+		const char *cached_existance = sdb_const_get (fs->fscache, file, NULL);
 		bool file_exists = false;
 		if (cached_existance) {
 			file_exists = !strcmp (cached_existance, "1");
 		} else {
 			if (r_file_exists (file)) {
-				sdb_set (fscache, file, "1", 0);
+				sdb_set (fs->fscache, file, "1", 0);
 			} else {
-				sdb_set (fscache, file, "0", 0);
+				sdb_set (fs->fscache, file, "0", 0);
 			}
 		}
 		if (file_exists) {
@@ -279,7 +258,80 @@ static bool print_addrinfo_json(void *user, const char *k, const char *v) {
 	return true;
 }
 
+static bool print_addrinfo2_json(void *user, RBinAddrline *item) {
+	FilterStruct *fs = (FilterStruct *)user;
+	ut64 offset = item->addr;
+	if (!offset || offset == UT64_MAX) {
+		return true;
+	}
+#if 0
+	if (colonpos && (fs->filter_offset == UT64_MAX || fs->filter_offset == offset)) {
+		if (fs->filter_format) {
+			*colonpos = ':';
+	//		r_cons_printf (core->cons, "CL %s %s\n", k, subst);
+		} else {
+			*colonpos = 0;
+	//		r_cons_printf (core->cons, "file: %s\nline: %s\naddr: 0x%08"PFMT64x"\n", subst, colonpos + 1, offset);
+		}
+		fs->filter_count++;
+	}
+#endif
+	const char *file = item->file;
+	int line = item->line;
+	PJ *pj = fs->pj;
+	if (pj) {
+		pj_o (pj);
+		pj_ks (pj, "file", item->file);
+		pj_kn (pj, "line", item->line);
+		if (item->column > 0) {
+			pj_kn (pj, "column", item->column);
+		}
+		pj_kn (pj, "addr", item->addr);
+		const char *cached_existance = sdb_const_get (fs->fscache, file, NULL);
+		bool file_exists = false;
+		if (cached_existance) {
+			file_exists = !strcmp (cached_existance, "1");
+		} else {
+			if (r_file_exists (file)) {
+				sdb_set (fs->fscache, file, "1", 0);
+			} else {
+				sdb_set (fs->fscache, file, "0", 0);
+			}
+		}
+		if (file_exists) {
+			char *row = r_file_slurp_line (file, line, 0);
+			pj_ks (pj, "text", file);
+			free (row);
+		}
+		pj_end (pj);
+	}
+	return true;
+}
+
+static bool print_addrinfo2(void *user, RBinAddrline *item) {
+	FilterStruct *fs = (FilterStruct*)user;
+	ut64 offset = item->addr;
+	if (!offset || offset == UT64_MAX) {
+		return true;
+	}
+	if (fs->filter_offset == UT64_MAX || fs->filter_offset == offset) {
+		if (fs->filter_format) {
+			// TODO add column if defined
+			r_cons_printf (fs->core->cons, "'CL 0x%08"PFMT64x" %s:%d\n", item->addr, item->file, item->line);
+		} else {
+			r_cons_printf (fs->core->cons, "file: %s\nline: %d\ncolu: %d\naddr: 0x%08"PFMT64x"\n",
+				item->file, item->line, item->column, item->addr);
+		}
+		fs->filter_count++;
+	}
+	// TODO: return false if filter_offset is found ?
+
+	return true;
+}
+
+// R2_600 - DEPRECATE
 static bool print_addrinfo(void *user, const char *k, const char *v) {
+	FilterStruct *fs = (FilterStruct*)user;
 	ut64 offset = sdb_atoi (k);
 	if (!offset || offset == UT64_MAX) {
 		return true;
@@ -290,11 +342,11 @@ static bool print_addrinfo(void *user, const char *k, const char *v) {
 		colonpos = strchr (subst, ':'); // : for shell and | for db.. imho : everywhere
 	}
 	if (!colonpos) {
-		r_cons_printf ("%s\n", subst);
-	} else if (filter_offset == UT64_MAX || filter_offset == offset) {
-		if (filter_format) {
+		r_cons_printf (fs->core->cons, "%s\n", subst);
+	} else if (fs->filter_offset == UT64_MAX || fs->filter_offset == offset) {
+		if (fs->filter_format) {
 			*colonpos = ':';
-			r_cons_printf ("\"\"CL %s %s\n", k, subst);
+			r_cons_printf (fs->core->cons, "'CL %s %s\n", k, subst);
 		} else {
 			*colonpos++ = 0;
 			int line = atoi (colonpos);
@@ -304,18 +356,32 @@ static bool print_addrinfo(void *user, const char *k, const char *v) {
 				*columnpos ++ = 0;
 				colu = atoi (columnpos);
 			}
-
-			r_cons_printf ("file: %s\nline: %d\ncolu: %d\naddr: 0x%08"PFMT64x"\n",
+			r_cons_printf (fs->core->cons, "file: %s\nline: %d\ncolu: %d\naddr: 0x%08"PFMT64x"\n",
 				subst, line, colu, offset);
 		}
-		filter_count++;
+		fs->filter_count++;
 	}
 	free (subst);
 
 	return true;
 }
 
-static int cmd_meta_add_fileline(Sdb *s, const char *fileline, ut64 offset) {
+static int cmd_meta_add_fileline(RBinFile *bf, const char *fileline, ut64 offset) {
+#if 1
+	char *file = strdup (fileline);
+	char *line = strchr (file, ':');
+	if (line) {
+		*line++ = 0;
+	}
+	RBinAddrline item = {
+		.addr = offset,
+		.file = file,
+		.line = line? atoi (line): 0,
+	};
+	bf->addrline.al_add (&bf->addrline, item);
+	free (file);
+#else
+	Sdb *s = bf->sdb_addrinfo;
 	char aoffset[SDB_NUM_BUFSZ];
 	char *aoffsetptr = sdb_itoa (offset, 16, aoffset, sizeof (aoffset));
 	if (!aoffsetptr) {
@@ -327,6 +393,7 @@ static int cmd_meta_add_fileline(Sdb *s, const char *fileline, ut64 offset) {
 	if (!sdb_add (s, fileline, aoffsetptr, 0)) {
 		sdb_set (s, fileline, aoffsetptr, 0);
 	}
+#endif
 	return 0;
 }
 
@@ -339,6 +406,8 @@ static int cmd_meta_lineinfo(RCore *core, const char *input) {
 	const char *p = input;
 	char *file_line = NULL;
 
+	FilterStruct fs = { core, UT64_MAX, 0, 0, NULL };
+
 	if (*p == '?') {
 		r_core_cmd_help (core, help_msg_CL);
 		return 0;
@@ -349,25 +418,26 @@ static int cmd_meta_lineinfo(RCore *core, const char *input) {
 			// same as CLL@@i = r_core_cmd0 (core, "list");
 			return 0;
 		}
-		ut64 at = core->offset;
+		ut64 at = core->addr;
 		if (p[1] == ' ') {
 			at = r_num_get (core->num, p + 2);
 		}
-		char *text = r_bin_addr2text (core->bin, at, 0);
+		char *text = r_bin_addrline_tostring (core->bin, at, 0);
 		if (R_STR_ISNOTEMPTY (text)) {
-			r_cons_printf ("0x%08"PFMT64x"  %s\n", at, text);
+			r_cons_printf (core->cons, "0x%08"PFMT64x"  %s\n", at, text);
 		}
+		free (text);
 		return 0;
 	}
 	if (*p == 'f') { // "CLf"
 		int retries = 5;
-		ut64 addr = core->offset;
+		ut64 addr = core->addr;
 		if (p[1] == ' ') {
 			addr = r_num_math (core->num, p + 1);
 		}
 retry:
 		;
-		char *s = r_bin_addr2fileline (core->bin, addr);
+		char *s = r_bin_addrline_tostring (core->bin, addr, 3);
 		if (!s) {
 			RAnalOp *op = r_core_anal_op (core, addr, 0);
 			if (op) {
@@ -380,7 +450,7 @@ retry:
 		}
 		if (s) {
 			r_str_after (s, ':');
-			r_cons_println (s);
+			r_cons_println (core->cons, s);
 			free (s);
 			r_core_return_code (core, 0);
 		} else {
@@ -398,15 +468,18 @@ retry:
 	}
 	if (*p == '.') { // "CL."
 		p++;
-		offset = core->offset;
+		offset = core->addr;
 	}
 	if (*p == '+') { // "CL+"
-		offset = core->offset;
-		p = strdup (r_str_trim_head_ro (p + 1));
+		offset = core->addr;
+		p = r_str_trim_head_ro (p + 1);
 		RBinFile *bf = r_bin_cur (core->bin);
-		ret = cmd_meta_add_fileline (bf->sdb_addrinfo, p, offset);
+		if (bf) {
+			ret = cmd_meta_add_fileline (bf, p, offset);
+		}
 		return 0;
-	} else if (*p == ' ') { // "CL "
+	}
+	if (*p == ' ') { // "CL "
 		p = r_str_trim_head_ro (p + 1);
 		char *arg = strchr (p, ' ');
 		if (!arg) {
@@ -416,16 +489,19 @@ retry:
 	} else if (*p == '*') {
 		p++;
 		all = true;
-		filter_format = '*';
+		fs.filter_format = '*';
 	} else {
-		filter_format = 0;
+		fs.filter_format = 0;
 	}
-
 	if (all && core->bin->cur) {
 		if (remove) {
-			sdb_reset (core->bin->cur->sdb_addrinfo);
+			r_bin_addrline_reset (core->bin);
 		} else {
-			sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, NULL);
+			if (core->bin->cur && core->bin->cur->addrline.used) {
+				r_bin_addrline_foreach (core->bin, print_addrinfo2, &fs);
+			} else {
+				sdb_foreach (core->bin->cur->sdb_addrinfo, print_addrinfo, &fs);
+			}
 		}
 		return 0;
 	}
@@ -451,11 +527,12 @@ retry:
 			sp = pheap = (char *)o;
 		}
 		RBinFile *bf = r_bin_cur (core->bin);
-		ret = 0;
-		if (bf && bf->sdb_addrinfo) {
-			ret = cmd_meta_add_fileline (bf->sdb_addrinfo, sp, offset);
+		if (bf) {
+			// R_LOG_ERROR ("deprecated way to add addrinfo metadata");
+			ret = cmd_meta_add_fileline (bf, sp, offset);
 		} else {
-			R_LOG_TODO ("Support global SdbAddrinfo or dummy rbinfile to handlee this case");
+			R_LOG_TODO ("Support global SdbAddrinfo or dummy rbinfile to handle this case");
+			ret = 0;
 		}
 		free (file_line);
 		free (myp);
@@ -464,49 +541,57 @@ retry:
 	}
 	free (myp);
 	if (remove) {
-		remove_meta_offset (core, offset);
+		r_bin_addrline_reset_at (core->bin, offset);
 	} else {
 		// taken from r2 // TODO: we should move this addrinfo sdb logic into RBin.. use HT
-		filter_offset = offset;
-		filter_count = 0;
-		fscache = sdb_new0 ();
+		fs.filter_offset = offset;
+		fs.filter_count = 0;
+		fs.fscache = sdb_new0 ();
 		PJ *pj = NULL;
 		RBinFile *bf = r_bin_cur (core->bin);
 		if (use_json) {
 			pj = r_core_pj_new (core);
+			fs.pj = pj;
 			pj_a (pj);
-			if (bf && bf->sdb_addrinfo) {
-				sdb_foreach (bf->sdb_addrinfo, print_addrinfo_json, pj);
+			if (!r_bin_addrline_foreach (core->bin, print_addrinfo2_json, &fs)) {
+				if (bf && bf->sdb_addrinfo) {
+					sdb_foreach (bf->sdb_addrinfo, print_addrinfo_json, &fs);
+				}
 			}
 		} else {
-			if (bf && bf->sdb_addrinfo) {
-				sdb_foreach (bf->sdb_addrinfo, print_addrinfo, NULL);
+			if (!r_bin_addrline_foreach (core->bin, print_addrinfo2, &fs)) {
+				if (bf && bf->sdb_addrinfo) {
+					sdb_foreach (bf->sdb_addrinfo, print_addrinfo, &fs);
+				}
 			}
 		}
-		if (filter_count == 0) {
+		if (fs.filter_count == 0) {
 			print_meta_offset (core, offset, pj);
 		}
 		if (use_json) {
 			pj_end (pj);
 			char *s = pj_drain (pj);
 			if (s) {
-				r_cons_printf ("%s\n", s);
+				r_cons_printf (core->cons, "%s\n", s);
 				free (s);
 			}
 		}
-		sdb_free (fscache);
+		sdb_free (fs.fscache);
 	}
 	return 0;
 }
 
 static int cmd_meta_comment(RCore *core, const char *input) {
-	ut64 addr = core->offset;
+	ut64 addr = core->addr;
 	switch (input[1]) {
 	case '?':
 		r_core_cmd_help (core, help_msg_CC);
 		break;
 	case ',': // "CC,"
-		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, ',', input + 2);
+		{
+			RTable *t = r_core_table_new (core, "meta");
+			r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, ',', input + 2, t);
+		}
 		break;
 	case 'F': // "CC,"
 		if (input[2] == '?') {
@@ -527,28 +612,28 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			}
 		} else {
 			const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, addr);
-			if (comment && *comment) {
+			if (R_STR_ISNOTEMPTY (comment)) {
 				char *cmtfile = r_str_between (comment, ",(", ")");
-				if (cmtfile && *cmtfile) {
+				if (R_STR_ISNOTEMPTY (cmtfile)) {
 					char *cwd = getcommapath (core);
-					r_cons_printf ("%s"R_SYS_DIR"%s\n", cwd, cmtfile);
+					r_cons_printf (core->cons, "%s"R_SYS_DIR"%s\n", cwd, cmtfile);
 					free (cwd);
 				}
 				free (cmtfile);
 			}
 		}
 		break;
-	case '.':
-		  {
-			  ut64 at = input[2]? r_num_math (core->num, input + 2): addr;
-			  const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, at);
-			  if (comment) {
-				  r_cons_println (comment);
-			  }
-		  }
+	case '.': // "CC."
+		{
+			ut64 at = input[2]? r_num_math (core->num, input + 2): addr;
+			const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, at);
+			if (R_STR_ISNOTEMPTY (comment)) {
+				r_cons_println (core->cons, comment);
+			}
+		}
 		break;
 	case 0: // "CC"
-		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 0, NULL);
+		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 0, NULL, NULL);
 		break;
 	case 'f': // "CCf"
 		switch (input[2]) {
@@ -556,7 +641,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			{
 				ut64 arg = r_num_math (core->num, input + 2);
 				if (!arg) {
-					arg = core->offset;
+					arg = core->addr;
 				}
 				RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, arg, 0);
 				if (fcn) {
@@ -573,21 +658,24 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			}
 			break;
 		case ',': // "CCf,"
-			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, ',', core->offset, input + 3);
+			{
+				RTable *t = r_core_table_new (core, "comments");
+				r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, ',', core->addr, input + 3, t);
+			}
 			break;
 		case 'j': // "CCfj"
-			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 'j', core->offset, NULL);
+			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 'j', core->addr, NULL, NULL);
 			break;
 		case '*': // "CCf*"
-			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 1, core->offset, NULL);
+			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 1, core->addr, NULL, NULL);
 			break;
 		default:
-			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 0, core->offset, NULL);
+			r_meta_print_list_in_function (core->anal, R_META_TYPE_COMMENT, 0, core->addr, NULL, NULL);
 			break;
 		}
 		break;
 	case 'j': // "CCj"
-		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 'j', input + 2);
+		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 'j', input + 2, NULL);
 		break;
 	case '!': // "CC!"
 		{
@@ -618,27 +706,19 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			}
 		}
 		const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, addr);
-		char *text;
 		char *nc = strdup (newcomment);
 		r_str_unescape (nc);
 		r_str_ansi_strip (nc);
 		if (comment) {
-			text = malloc (strlen (comment) + strlen (newcomment) + 2);
-			if (text) {
-				strcpy (text, comment);
-				strcat (text, " ");
-				strcat (text, nc);
-				r_meta_set_string (core->anal, R_META_TYPE_COMMENT, addr, text);
-				free (text);
-			} else {
-				r_sys_perror ("malloc");
-			}
+			char *text = r_str_newf ("%s %s", comment, nc);
+			r_meta_set_string (core->anal, R_META_TYPE_COMMENT, addr, text);
+			free (text);
 		} else {
 			r_meta_set_string (core->anal, R_META_TYPE_COMMENT, addr, nc);
 			if (r_config_get_b (core->config, "cmd.undo")) {
-				char *a = r_str_newf ("CC-0x%08"PFMT64x, addr);
-				char *b = r_str_newf ("CC %s@0x%08"PFMT64x, nc, addr);
-				RCoreUndo *uc = r_core_undo_new (core->offset, b, a);
+				char *a = r_str_newf ("'CC-0x%08"PFMT64x, addr);
+				char *b = r_str_newf ("'@0x%08"PFMT64x"'CC %s", addr, nc);
+				RCoreUndo *uc = r_core_undo_new (core->addr, b, a);
 				r_core_undo_push (core, uc);
 				free (a);
 				free (b);
@@ -648,7 +728,7 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		}
 		break;
 	case '*': // "CC*"
-		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 1, NULL);
+		r_meta_print_list_all (core->anal, R_META_TYPE_COMMENT, 1, NULL, NULL);
 		break;
 	case '-': // "CC-"
 		if (input[2] == '*') { // "CC-*"
@@ -657,25 +737,17 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 			ut64 arg = r_num_math (core->num, input + 2);
 			r_meta_del (core->anal, R_META_TYPE_COMMENT, arg, 1);
 		} else { // "CC-"
-			r_meta_del (core->anal, R_META_TYPE_COMMENT, core->offset, 1);
+			r_meta_del (core->anal, R_META_TYPE_COMMENT, core->addr, 1);
 		}
 		break;
 	case 'u': // "CCu"
-		//
 		{
-		char *newcomment;
 		const char *arg = input + 2;
-		while (*arg && *arg == ' ') arg++;
-		if (r_str_startswith (arg, "base64:")) {
-			char *s = (char *)sdb_decode (arg + 7, NULL);
-			if (s) {
-				newcomment = s;
-			} else {
-				newcomment = NULL;
-			}
-		} else {
-			newcomment = strdup (arg);
+		while (*arg && *arg == ' ') {
+			arg++;
 		}
+		char *newcomment = r_str_startswith (arg, "base64:")
+			? (char *)sdb_decode (arg + 7, NULL): strdup (arg);
 		if (newcomment) {
 			const char *comment = r_meta_get_string (core->anal, R_META_TYPE_COMMENT, addr);
 			if (!comment || (comment && !strstr (comment, newcomment))) {
@@ -687,15 +759,14 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 		break;
 	case 'a': // "CCa"
 		{
-		char *s, *p;
-		s = strchr (input, ' ');
+		char *s = strchr (input, ' ');
 		if (s) {
 			s = strdup (s + 1);
 		} else {
 			r_core_cmd_help_match (core, help_msg_CC, "CCa");
 			return false;
 		}
-		p = strchr (s, ' ');
+		char *p = strchr (s, ' ');
 		if (p) {
 			*p++ = 0;
 		}
@@ -738,13 +809,13 @@ static int cmd_meta_comment(RCore *core, const char *input) {
 }
 
 static int cmd_meta_vartype_comment(RCore *core, const char *input) {
-	ut64 addr = core->offset;
+	ut64 addr = core->addr;
 	switch (input[1]) {
 	case '?': // "Ct?"
 		r_core_cmd_help (core, help_msg_Ct);
 		break;
 	case 0: // "Ct"
-		r_meta_print_list_all (core->anal, R_META_TYPE_VARTYPE, 0, NULL);
+		r_meta_print_list_all (core->anal, R_META_TYPE_VARTYPE, 0, NULL, NULL);
 		break;
 	case ' ': // "Ct <vartype comment> @ addr"
 		{
@@ -754,11 +825,9 @@ static int cmd_meta_vartype_comment(RCore *core, const char *input) {
 		r_str_unescape (nc);
 		if (comment) {
 			char *text = r_str_newf ("%s %s", comment, nc);
-			if (text) {
+			if (R_LIKELY (text)) {
 				r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, addr, text);
 				free (text);
-			} else {
-				r_sys_perror ("malloc");
 			}
 		} else {
 			r_meta_set_string (core->anal, R_META_TYPE_VARTYPE, addr, nc);
@@ -770,19 +839,18 @@ static int cmd_meta_vartype_comment(RCore *core, const char *input) {
 		{
 		ut64 at = input[2]? r_num_math (core->num, input + 2): addr;
 		const char *comment = r_meta_get_string (core->anal, R_META_TYPE_VARTYPE, at);
-		if (comment) {
-			r_cons_println (comment);
+		if (R_STR_ISNOTEMPTY (comment)) {
+			r_cons_println (core->cons, comment);
 		}
 		}
 		break;
 	case '-': // "Ct-"
-		r_meta_del (core->anal, R_META_TYPE_VARTYPE, core->offset, 1);
+		r_meta_del (core->anal, R_META_TYPE_VARTYPE, core->addr, 1);
 		break;
 	default:
 		r_core_cmd_help (core, help_msg_Ct);
 		break;
 	}
-
 	return true;
 }
 
@@ -793,15 +861,15 @@ typedef struct {
 	int bufsz;
 } StringSearchOptions;
 
-static int cb_strhit(R_NULLABLE RSearchKeyword *kw, void *user, ut64 where) {
+static int cb_strhit(RSearchKeyword * R_NULLABLE kw, void *user, ut64 where) {
 	StringSearchOptions *sso = (StringSearchOptions*)user;
 	if (where - sso->addr >= sso->bufsz) {
-		r_core_cmdf (sso->core, "Cz@0x%08"PFMT64x, where);
+		r_core_cmd_call_at (sso->core, where, "Cz");
 	} else {
 		const char *name = (const char *)(sso->buf + (where - sso->addr));
-		size_t maxlen = sso->bufsz - (where - sso->addr);
+		const size_t maxlen = sso->bufsz - (where - sso->addr);
 		char *hname = r_str_ndup (name, maxlen);
-		size_t n = strlen (hname) + 1;
+		const size_t n = strlen (hname) + 1;
 		r_meta_set (sso->core->anal, R_META_TYPE_STRING, where, n, hname);
 		free (hname);
 	}
@@ -811,7 +879,7 @@ static int cb_strhit(R_NULLABLE RSearchKeyword *kw, void *user, ut64 where) {
 static int cmd_meta_others(RCore *core, const char *input) {
 	char *t = 0, *p, *p2, name[256] = {0};
 	int n, repeat = 1;
-	ut64 addr = core->offset;
+	ut64 addr = core->addr;
 
 	int type = input[0];
 	if (!type) {
@@ -827,7 +895,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 		switch (input[0]) {
 		case 'f': // "Cf?"
 			r_core_cmd_help_match (core, help_msg_C, "Cf");
-			r_cons_println (
+			r_cons_println (core->cons,
 				"'sz' indicates the byte size taken up by struct.\n"
 				"'fmt' is a 'pf?' style format string. It controls only the display format.\n\n"
 				"You may wish to have 'sz' != sizeof (fmt) when you have a large struct\n"
@@ -839,7 +907,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			r_core_cmd_help (core, help_msg_Cs);
 			break;
 		default:
-			r_cons_println ("See C?");
+			r_cons_println (core->cons, "See C?");
 			break;
 		}
 		break;
@@ -872,10 +940,10 @@ static int cmd_meta_others(RCore *core, const char *input) {
 		}
 		break;
 	case '*': // "Cf*", "Cd*", ...
-		r_meta_print_list_all (core->anal, input[0], 1, NULL);
+		r_meta_print_list_all (core->anal, input[0], 1, NULL, NULL);
 		break;
 	case 'j': // "Cfj", "Cdj", ...
-		r_meta_print_list_all (core->anal, input[0], 'j', NULL);
+		r_meta_print_list_all (core->anal, input[0], 'j', NULL, NULL);
 		break;
 	case '!': // "Cf!", "Cd!", ...
 		{
@@ -896,15 +964,15 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			ut64 size;
 			RAnalMetaItem *mi = r_meta_get_at (core->anal, addr, type, &size);
 			if (mi) {
-				r_meta_print (core->anal, mi, addr, size, input[3], NULL, false);
+				r_meta_print (core->anal, mi, addr, size, input[3], NULL, NULL, false);
 			}
 			break;
 		} else if (input[2] == 'j') { // "Cs.j"
 			ut64 size;
 			RAnalMetaItem *mi = r_meta_get_at (core->anal, addr, type, &size);
 			if (mi) {
-				r_meta_print (core->anal, mi, addr, size, input[2], NULL, false);
-				r_cons_newline ();
+				r_meta_print (core->anal, mi, addr, size, input[2], NULL, NULL, false);
+				r_cons_newline (core->cons);
 			}
 			break;
 		}
@@ -920,21 +988,23 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			case R_STRING_ENC_UTF8:
 				esc_str = r_str_escape_utf8 (mi->str, false, esc_bslash);
 				break;
-			case 0:  /* temporary legacy workaround */
+			case 0: // temporary legacy workaround
 				esc_bslash = false;
+				// falltrhru
 			default:
 				esc_str = r_str_escape_latin1 (mi->str, false, esc_bslash, false);
+				break;
 			}
 			if (esc_str) {
-				r_cons_printf ("\"%s\"\n", esc_str);
+				r_cons_printf (core->cons, "\"%s\"\n", esc_str);
 				free (esc_str);
 			} else {
-				r_cons_println ("<oom>");
+				r_cons_println (core->cons, "<oom>");
 			}
 		} else if (type == 'd') {
-			r_cons_printf ("%"PFMT64u"\n", size);
+			r_cons_printf (core->cons, "%"PFMT64u"\n", size);
 		} else {
-			r_cons_println (mi->str);
+			r_cons_println (core->cons, mi->str);
 		}
 		break;
 	case 's': // "Css"
@@ -945,9 +1015,9 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			}
 			if (range == UT64_MAX || range == 0) {
 				// get cursection size
-				RBinSection *s = r_bin_get_section_at (r_bin_cur_object (core->bin), core->offset, true);
+				RBinSection *s = r_bin_get_section_at (r_bin_cur_object (core->bin), core->addr, true);
 				if (s) {
-					range = s->vaddr + s->vsize - core->offset;
+					range = s->vaddr + s->vsize - core->addr;
 				}
 				// TODO use debug maps if cfg.debug=true?
 			}
@@ -959,15 +1029,14 @@ static int cmd_meta_others(RCore *core, const char *input) {
 				ut8 *buf = malloc (range + 1);
 				if (buf) {
 					buf[range] = 0;
-					const ut64 addr = core->offset;
+					const ut64 addr = core->addr;
 					const int minstr = r_config_get_i (core->config, "bin.str.min");
 					const int maxstr = r_config_get_i (core->config, "bin.str.max");
 					r_core_cmdf (core, "Cz@0x%08"PFMT64x, addr);
 					// maps are not yet set
-					char *s = r_core_cmd_str (core, "o;om");
-					free (s);
+					free (r_core_cmd_str (core, "o;om")); // wtf?
 					if (!r_io_read_at (core->io, addr, buf, range)) {
-						R_LOG_ERROR ("cannot read %d", range);
+						R_LOG_ERROR ("Cannot read %d", range);
 					}
 					RSearch *ss = r_search_new (R_SEARCH_STRING);
 					r_search_set_string_limits (ss, minstr, maxstr);
@@ -986,10 +1055,9 @@ static int cmd_meta_others(RCore *core, const char *input) {
 				} else {
 					R_LOG_ERROR ("Cannot allocate");
 				}
-
 #if 0
 				r_core_cmdf (core, "/z 8 100@0x%08"PFMT64x"@e:search.in=range@e:search.from=0x%"PFMT64x"@e:search.to=0x%"PFMT64x,
-						core->offset, core->offset, core->offset + range);
+						core->addr, core->addr, core->addr + range);
 				r_core_cmd0 (core, "Csz @@ hit*;f-hit*");
 #else
 #endif
@@ -1022,7 +1090,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 			type = 's';
 		} else {
 			if (!input[1] && !core->tmpseek) {
-				r_meta_print_list_all (core->anal, type, 0, NULL);
+				r_meta_print_list_all (core->anal, type, 0, NULL, NULL);
 				break;
 			}
 		}
@@ -1199,7 +1267,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 						r_str_ncpy (name, p, sizeof (name));
 					} else {
 						if (type != 'b' && type != 's') {
-							RFlagItem *fi = r_flag_get_i (core->flags, addr);
+							RFlagItem *fi = r_flag_get_in (core->flags, addr);
 							if (fi) {
 								r_str_ncpy (name, fi->name, sizeof (name));
 							}
@@ -1231,7 +1299,6 @@ static int cmd_meta_others(RCore *core, const char *input) {
 		// r_meta_cleanup (core->anal->meta, 0LL, UT64_MAX);
 		break;
 	default:
-		eprintf ("((((%s))))\n", input);
 		R_LOG_ERROR ("Missing space after CC");
 		break;
 	}
@@ -1239,7 +1306,7 @@ static int cmd_meta_others(RCore *core, const char *input) {
 	return true;
 }
 
-void r_comment_var_help(RCore *core, char type) {
+static void comment_var_help(RCore *core, char type) {
 	switch (type) {
 	case 'b':
 		r_core_cmd_help (core, help_msg_Cvb);
@@ -1251,17 +1318,17 @@ void r_comment_var_help(RCore *core, char type) {
 		r_core_cmd_help (core, help_msg_Cvr);
 		break;
 	case '?':
-		r_cons_printf("See Cvb?, Cvs? and Cvr?\n");
+		r_cons_printf (core->cons, "See Cvb?, Cvs? and Cvr?\n");
 	}
 }
 
-void r_comment_vars(RCore *core, const char *input) {
-	//TODO enable base64 and make it the default for C*
-	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->offset, 0);
+static void cmd_Cv(RCore *core, const char *input) {
+	// TODO enable base64 and make it the default for C*
+	RAnalFunction *fcn = r_anal_get_fcn_in (core->anal, core->addr, 0);
 	char *oname = NULL, *name = NULL;
 
 	if (!input[0] || input[1] == '?' || (input[0] != 'b' && input[0] != 'r' && input[0] != 's')) {
-		r_comment_var_help (core, input[0]);
+		comment_var_help (core, input[0]);
 		return;
 	}
 	if (!fcn) {
@@ -1279,14 +1346,15 @@ void r_comment_vars(RCore *core, const char *input) {
 			if (var->kind != kind || !var->comment) {
 				continue;
 			}
-			if (!input[1]) {
-				r_cons_printf ("%s : %s\n", var->name, var->comment);
-			} else {
+			if (input[1]) {
 				char *b64 = sdb_encode ((const ut8 *)var->comment, strlen (var->comment));
 				if (!b64) {
 					continue;
 				}
-				r_cons_printf ("\"Cv%c %s base64:%s @ 0x%08"PFMT64x"\"\n", kind, var->name, b64, fcn->addr);
+				r_cons_printf (core->cons, "'@0x%08"PFMT64x"'Cv%c %s base64:%s\n", fcn->addr, kind, var->name, b64);
+				free (b64);
+			} else {
+				r_cons_printf (core->cons, "%s : %s\n", var->name, var->comment);
 			}
 		}
 		}
@@ -1305,23 +1373,23 @@ void r_comment_vars(RCore *core, const char *input) {
 		}
 		RAnalVar *var = r_anal_function_get_var_byname (fcn, name);
 		if (!var) {
-			int idx = (int)strtol (name, NULL, 0);
+			const int idx = (int)strtol (name, NULL, 0);
 			var = r_anal_function_get_var (fcn, input[0], idx);
 		}
-		if (!var) {
-			R_LOG_ERROR ("can't find variable at given offset");
-		} else {
+		if (var) {
 			if (var->comment) {
-				if (comment && *comment) {
+				if (R_STR_ISNOTEMPTY (comment)) {
 					char *text = r_str_newf ("%s\n%s", var->comment, comment);
 					free (var->comment);
 					var->comment = text;
 				} else {
-					r_cons_println (var->comment);
+					r_cons_println (core->cons, var->comment);
 				}
-			} else {
+			} else if (R_STR_ISNOTEMPTY (comment)) {
 				var->comment = strdup (comment);
 			}
+		} else {
+			R_LOG_ERROR ("can't find variable at given offset");
 		}
 		free (heap_comment);
 		}
@@ -1370,27 +1438,27 @@ static int cmd_meta(void *data, const char *input) {
 
 	switch (*input) {
 	case 'v': // "Cv"
-		r_comment_vars (core, input + 1);
+		cmd_Cv (core, input + 1);
 		break;
 	case '\0': // "C"
-		r_meta_print_list_all (core->anal, R_META_TYPE_ANY, 0, NULL);
+		r_meta_print_list_all (core->anal, R_META_TYPE_ANY, 0, NULL, NULL);
 		break;
 	case ',': // "C,"
-		r_meta_print_list_all (core->anal, R_META_TYPE_ANY, *input, input + 1);
+		r_meta_print_list_all (core->anal, R_META_TYPE_ANY, *input, input + 1, NULL);
 		break;
 	case 'j': // "Cj"
 	case '*': { // "C*"
 		if (input[1] == '.') {
-			r_meta_print_list_at (core->anal, core->offset, *input, input + 2);
+			r_meta_print_list_at (core->anal, core->addr, *input, input + 2, NULL);
 		} else if (input[1]) {
-			r_meta_print_list_at (core->anal, core->offset, *input, input + 2);
+			r_meta_print_list_at (core->anal, core->addr, *input, input + 2, NULL);
 		} else {
-			r_meta_print_list_all (core->anal, R_META_TYPE_ANY, *input, input + 1);
+			r_meta_print_list_all (core->anal, R_META_TYPE_ANY, *input, input + 1, NULL);
 		}
 		break;
 	}
 	case '.': { // "C."
-		r_meta_print_list_at (core->anal, core->offset, 0, NULL);
+		r_meta_print_list_at (core->anal, core->addr, 0, NULL, NULL);
 		break;
 	}
 	case 'L': // "CL"
@@ -1415,7 +1483,7 @@ static int cmd_meta(void *data, const char *input) {
 	case '-': // "C-"
 		if (input[1] != '*') {
 			i = input[1] ? r_num_math (core->num, input + (input[1] == ' ' ? 2 : 1)) : 1;
-			r_meta_del (core->anal, R_META_TYPE_ANY, core->offset, i);
+			r_meta_del (core->anal, R_META_TYPE_ANY, core->addr, i);
 		} else {
 			r_meta_del (core->anal, R_META_TYPE_ANY, 0, UT64_MAX);
 		}
@@ -1424,7 +1492,7 @@ static int cmd_meta(void *data, const char *input) {
 		r_core_cmd_help (core, help_msg_C);
 		break;
 	case 'F': // "CF"
-		f = r_anal_get_fcn_in (core->anal, core->offset,
+		f = r_anal_get_fcn_in (core->anal, core->addr,
 			R_ANAL_FCN_TYPE_FCN|R_ANAL_FCN_TYPE_SYM);
 		if (f) {
 			r_anal_str_to_fcn (core->anal, f, input + 2);
