@@ -41,33 +41,34 @@ R_API R_MUSTUSE const RFSType* r_fs_type_index(int i) {
 }
 
 R_API R_MUSTUSE RFS* r_fs_new(void) {
-	int i;
-	RFSPlugin* static_plugin;
-	RFS* fs = R_NEW0 (RFS);
-	if (fs) {
-		fs->view = R_FS_VIEW_NORMAL;
-		fs->roots = r_list_new ();
-		if (!fs->roots) {
-			r_fs_free (fs);
-			return NULL;
+	RFSPlugin *static_plugin;
+	RFS *fs = R_NEW0 (RFS);
+	fs->view = R_FS_VIEW_NORMAL;
+	fs->roots = r_list_new ();
+	if (!fs->roots) {
+		r_fs_free (fs);
+		return NULL;
+	}
+	fs->roots->free = (RListFree) r_fs_root_free;
+	fs->plugins = r_list_new ();
+	if (!fs->plugins) {
+		r_fs_free (fs);
+		return NULL;
+	}
+	fs->plugins->free = free;
+	// XXX fs->roots->free = r_fs_plugin_free;
+	size_t i;
+	for (i = 0; fs_static_plugins[i]; i++) {
+		if (!fs_static_plugins[i]->meta.name) {
+			continue;
 		}
-		fs->roots->free = (RListFree) r_fs_root_free;
-		fs->plugins = r_list_new ();
-		if (!fs->plugins) {
-			r_fs_free (fs);
-			return NULL;
-		}
-		fs->plugins->free = free;
-		// XXX fs->roots->free = r_fs_plugin_free;
-		for (i = 0; fs_static_plugins[i]; i++) {
-			static_plugin = R_NEW (RFSPlugin);
-			if (!static_plugin) {
-				continue;
-			}
-			memcpy (static_plugin, fs_static_plugins[i], sizeof (RFSPlugin));
-			r_fs_plugin_add (fs, static_plugin);
-			free (static_plugin);
+		static_plugin = R_NEW (RFSPlugin);
+		if (!static_plugin) {
+			continue;
 		}
+		memcpy (static_plugin, fs_static_plugins[i], sizeof (RFSPlugin));
+		r_fs_plugin_add (fs, static_plugin);
+		free (static_plugin);
 	}
 	return fs;
 }
@@ -98,15 +99,15 @@ R_API void r_fs_free(RFS* fs) {
 R_API bool r_fs_plugin_add(RFS* fs, RFSPlugin* p) {
 	R_RETURN_VAL_IF_FAIL (fs && p, false);
 	if (p->init) {
-		p->init ();
+		// TODO. return false if init fails?
+		if (!p->init ()) {
+			return false;
+		}
 	}
 	RFSPlugin* sp = R_NEW0 (RFSPlugin);
-	if (sp) {
-		memcpy (sp, p, sizeof (RFSPlugin));
-		r_list_append (fs->plugins, sp);
-		return true;
-	}
-	return false;
+	memcpy (sp, p, sizeof (RFSPlugin));
+	r_list_append (fs->plugins, sp);
+	return true;
 }
 
 R_API bool r_fs_plugin_remove(RFS *fs, RFSPlugin *p) {
@@ -119,9 +120,8 @@ R_API void r_fs_del(RFS* fs, RFSPlugin* p) {
 }
 
 /* mountpoint */
-R_API RFSRoot* r_fs_mount(RFS* fs, const char* fstype, const char* path, ut64 delta) {
-	R_RETURN_VAL_IF_FAIL (fs && fstype && path, NULL);
-	RFSPlugin* p;
+R_API RFSRoot* r_fs_mount(RFS* fs, const char * R_NULLABLE fstype, const char* path, ut64 delta) {
+	R_RETURN_VAL_IF_FAIL (fs && path, NULL);
 	RFSRoot* root;
 	RListIter* iter;
 	char* str;
@@ -129,15 +129,19 @@ R_API RFSRoot* r_fs_mount(RFS* fs, const char* fstype, const char* path, ut64 de
 	char *heapFsType = NULL;
 
 	if (path[0] != '/') {
-		R_LOG_ERROR ("invalid mountpoint %s", path);
+		R_LOG_ERROR ("Invalid mountpoint %s", path);
 		return NULL;
 	}
-	if (!fstype || !*fstype) {
+	if (R_STR_ISEMPTY (fstype)) {
 		heapFsType = r_fs_name (fs, delta);
 		fstype = (const char *)heapFsType;
 	}
-	if (!(p = r_fs_plugin_get (fs, fstype))) {
-		R_LOG_ERROR ("Invalid filesystem type");
+	if (fstype == NULL) {
+		return NULL;
+	}
+	RFSPlugin* p = r_fs_plugin_get (fs, fstype);
+	if (!p) {
+		R_LOG_ERROR ("Invalid filesystem type '%s'", fstype);
 		free (heapFsType);
 		return NULL;
 	}
