@@ -1,57 +1,84 @@
-/* radare - LGPL - Copyright 2009-2024 - pancake, nibble, maijin */
+/* radare - LGPL - Copyright 2009-2025 - pancake, nibble, maijin */
 
 #define R_LOG_ORIGIN "rasm2"
 
 #include <r_asm.h>
-#include <r_lib.h>
 #include <r_main.h>
 
+typedef struct {
+	bool oneliner;
+	bool coutput;
+	bool quiet;
+	bool envhelp;
+	bool json;
+	bool use_spp;
+	bool isbig;
+	bool rad;
+	bool bin;
+	const char *kernel;
+	const char *cpu;
+	int bits;
+} RAsmOptions;
+
 typedef struct {
 	RLib *l;
 	RAsm *a;
 	RAnal *anal;
-	bool oneliner;
-	bool coutput;
-	bool json;
-	bool quiet;
+	RAsmOptions opt;
 } RAsmState;
 
-static void __load_plugins(RAsmState *as);
+typedef struct {
+	const char *name;
+	const char *desc;
+} RAsmEnv;
 
-static void __as_set_archbits(RAsmState *as) {
-	const char *arch = as->a->config->arch; // R_SYS_ARCH;
+static RAsmEnv env[] = {
+	{ "R2_NOPLUGINS",		"do not load shared plugins (speedup loading)" },
+	{ "R2_LOG_LEVEL",		"change the log level" },
+	{ "R2_DEBUG",			"if defined, show error messages and crash signal" },
+	{ "R2_DEBUG_ASSERT",	"lldb -- r2 to get proper backtrace of the runtime assert" },
+	{ "RASM2_ARCH",			"same as rasm2 -a" },
+	{ "RASM2_BITS",			"same as rasm2 -b" }
+};
+
+static void rasm_load_plugins(RAsmState *as);
+static void rasm_show_env(bool show_desc);
+
+static void rasm_set_archbits(RAsmState *as) {
+	const char *arch = as->a->config->arch;
 	r_asm_use (as->a, arch);
 	r_anal_use (as->anal, arch);
-	int sysbits = (R_SYS_BITS & R_SYS_BITS_64)? 64: 32;
+	const int sysbits = R_SYS_BITS_CHECK (R_SYS_BITS, 64)? 64: 32;
 	r_asm_set_bits (as->a, sysbits);
 	r_anal_set_bits (as->anal, sysbits);
+	as->opt.bits = sysbits;
 }
 
-static RAsmState *__as_new(void) {
+static RAsmState *rasm_new(void) {
 	RAsmState *as = R_NEW0 (RAsmState);
-	if (as) {
-		as->l = r_lib_new (NULL, NULL);
-		as->a = r_asm_new ();
-		as->anal = r_anal_new ();
-		r_unref (as->anal->config);
-		as->a->num = r_num_new (NULL, NULL, NULL);
-		as->anal->config = r_ref_ptr (as->a->config);
-		r_anal_bind (as->anal, &as->a->analb);
-		const bool load_plugins = !r_sys_getenv_asbool ("R2_NOPLUGINS");
-		if (load_plugins) {
-			__load_plugins (as);
-		}
-		__as_set_archbits (as);
+	as->l = r_lib_new (NULL, NULL);
+	as->a = r_asm_new ();
+	as->anal = r_anal_new ();
+	r_unref (as->anal->config);
+	as->a->num = r_num_new (NULL, NULL, NULL);
+	as->anal->config = r_ref_ptr (as->a->config);
+	r_anal_bind (as->anal, &as->a->analb);
+	const bool load_plugins = !r_sys_getenv_asbool ("R2_NOPLUGINS");
+	if (load_plugins) {
+		rasm_load_plugins (as);
 	}
+	rasm_set_archbits (as);
 	return as;
 }
 
-static void __as_free(RAsmState *as) {
+static void rasm_free(RAsmState *as) {
 	if (as) {
 		if (as->a) {
 			r_num_free (as->a->num);
+			as->a->num = NULL;
+			r_asm_free (as->a);
 		}
-		r_asm_free (as->a);
+		// r_arch_free (as->anal->arch);
 		r_anal_free (as->anal);
 		// r_unref (as->a->config);
 		r_lib_free (as->l);
@@ -78,7 +105,7 @@ static int showanal(RAsmState *as, RAnalOp *op, ut64 offset, ut8 *buf, int len,
 	char *stackop = stackop2str (op->stackop);
 	const char *optype = r_anal_optype_tostring (op->type);
 	char *bytes = r_hex_bin2strdup (buf, ret);
-	if (as->json) {
+	if (as->opt.json) {
 		pj_o (pj);
 		pj_kn (pj, "opcode", offset);
 		pj_ks (pj, "bytes", bytes);
@@ -135,7 +162,7 @@ static int show_analinfo(RAsmState *as, const char *arg, ut64 offset) {
 		return 0;
 	}
 	RAnalOp aop = {0};
-	if (as->json) {
+	if (as->opt.json) {
 		pj_a (pj);
 	}
 	for (ret = 0; ret < len;) {
@@ -145,7 +172,7 @@ static int show_analinfo(RAsmState *as, const char *arg, ut64 offset) {
 			break;
 		}
 		if (aop.size < 1) {
-			if (as->json) {
+			if (as->opt.json) {
 				pj_o (pj);
 				pj_ks (pj, "bytes",  r_hex_bin2strdup (buf, ret));
 				pj_ks (pj, "type", "Invalid");
@@ -159,7 +186,7 @@ static int show_analinfo(RAsmState *as, const char *arg, ut64 offset) {
 		ret += aop.size;
 		r_anal_op_fini (&aop);
 	}
-	if (as->json) {
+	if (as->opt.json) {
 		pj_end (pj);
 		printf ("%s\n", pj_string (pj));
 		pj_free (pj);
@@ -179,11 +206,15 @@ static void rarch2_list(RAsmState *as, const char *arch) {
 	RArchPlugin *h;
 	RListIter *iter, *iter2;
 	PJ *pj = NULL;
-	if (as->json) {
+	if (as->opt.json) {
 		pj = pj_new ();
 		pj_a (pj);
 	}
-	r_list_foreach (as->anal->arch->plugins, iter, h) {
+	RList *plugins = R_UNWRAP4 (as, anal, arch, plugins);
+	r_list_foreach (plugins, iter, h) {
+		if (arch && strcmp (arch, h->meta.name)) {
+			continue;
+		}
 		const char *feat = "___";
 		if (h->encode) {
 			feat = h->decode? "ade": "a__";
@@ -194,20 +225,19 @@ static void rarch2_list(RAsmState *as, const char *arch) {
 		RList *bitslist = r_list_newf (NULL);
 		for (i = 0; i < 8; i++) {
 			ut8 bit = (bits & 0xFF); // TODO: use the macros
-			if (bit) {
-				r_list_append (bitslist, (void*)(size_t)bit);
-			} else {
+			if (!bit) {
 				break;
 			}
+			r_list_append (bitslist, (void*)(size_t)bit);
 			bits >>= 8; // TODO: use the macros
 		}
 		r_list_sort (bitslist, sizetsort);
 		char *bitstr = r_num_list_join (bitslist, " ");
-		if (as->quiet) {
+		if (as->opt.quiet) {
 			printf ("%s\n", h->meta.name);
-		} else if (as->json) {
+		} else if (as->opt.json) {
 			pj_o (pj);
-			pj_ks (pj, "name", h->meta.name);
+			r_lib_meta_pj (pj, &h->meta);
 			pj_k (pj, "bits");
 			pj_a (pj);
 			void *k;
@@ -215,25 +245,32 @@ static void rarch2_list(RAsmState *as, const char *arch) {
 				pj_i (pj, (int)(size_t)k);
 			}
 			pj_end (pj);
-			pj_ks (pj, "license", r_str_get_fail (h->meta.license, "unknown"));
-			pj_ks (pj, "description", h->meta.desc);
 			pj_ks (pj, "features", feat);
 			pj_end (pj);
-		} else {
-			printf ("%s %-11s %-11s %-7s %s", feat, bitstr, h->meta.name,
-				r_str_get_fail (h->meta.license, "unknown"), h->meta.desc);
+		} else if (arch) {
+			printf ("name: %s\n", h->meta.name);
+			printf ("bits: %s\n", bitstr);
+			printf ("desc: %s\n", h->meta.desc);
+			printf ("feat: %s\n", feat);
 			if (h->meta.author) {
-				printf (" (by %s)", h->meta.author);
+				printf ("auth: %s\n", h->meta.author);
+			}
+			if (h->meta.license) {
+				printf ("lice: %s\n", h->meta.license);
 			}
 			if (h->meta.version) {
-				printf (" v%s", h->meta.version);
+				printf ("vers: %s\n", h->meta.version);
 			}
-			printf ("\n");
+		} else {
+			printf ("%s %-11s %-11s %s\n", feat, bitstr, h->meta.name, h->meta.desc);
 		}
 		r_list_free (bitslist);
 		free (bitstr);
+		if (arch) {
+			break;
+		}
 	}
-	if (as->json) {
+	if (as->opt.json) {
 		pj_end (pj);
 		printf ("%s\n", pj_string (pj));
 	}
@@ -242,7 +279,7 @@ static void rarch2_list(RAsmState *as, const char *arch) {
 
 static int rasm_show_help(int v) {
 	if (v < 2) {
-		printf ("Usage: rasm2 [-ACdDehLBvw] [-a arch] [-b bits] [-s addr] [-S syntax]\n"
+		printf ("Usage: rasm2 [-ACdDehHLBvw] [-a arch] [-b bits] [-s addr] [-S syntax]\n"
 			"   [-f file] [-o file] [-F fil:ter] [-i skip] [-l len] 'code'|hex|0101b|-\n");
 	}
 	if (v != 1) {
@@ -256,16 +293,18 @@ static int rasm_show_help(int v) {
 			" -e           use big endian instead of little endian\n"
 			" -E           display ESIL expression (same input as in -d)\n"
 			" -f [file]    read data from file\n"
-			" -F [in:out]  specify input and/or output filters (att2intel, x86.pseudo, ..)\n"
+			" -F [parser]  specify which parse filter use (see -LL)\n" // TODO: rename to -p
 			" -h, -hh      show this help, -hh for long\n"
+			" -H ([var])   display variable\n"
 			" -i [len]     ignore/skip N bytes of the input buffer\n"
 			" -j           output in json format\n"
 			" -k [kernel]  select operating system (linux, windows, darwin, android, ios, ..)\n"
 			" -l [len]     input/Output length\n"
-			" -L           list RArch plugins: (a=asm, d=disasm, e=esil)\n"
+			" -L ([name])  list RArch plugins: (a=asm, d=disasm, e=esil)\n"
+			" -LL ([name]) list RAsm parse plugins\n"
 			" -N           same as r2 -N (or R2_NOPLUGINS) (not load any plugin)\n" // -n ?
 			" -o [file]    output file name (rasm2 -Bf a.asm -o a)\n"
-			" -p           run SPP over input for assembly\n"
+			" -p           run SPP over input for assembly\n" // TODO - must be done by default
 			" -q           quiet mode\n"
 			" -r           output in radare commands\n"
 			" -s,-@ [addr] define initial start/seek address (default 0)\n"
@@ -275,14 +314,8 @@ static int rasm_show_help(int v) {
 			" -w           what's this instruction for? describe opcode\n"
 			" If '-l' value is greater than output length, output is padded with nops\n"
 			" If the last argument is '-' reads from stdin\n");
-		printf ("Environment:\n"
-			" R2_NOPLUGINS   do not load shared plugins (speedup loading)\n"
-			" R2_LOG_LEVEL=X    change the log level\n"
-			" R2_DEBUG          if defined, show error messages and crash signal\n"
-			" R2_DEBUG_ASSERT=1 lldb -- r2 to get proper backtrace of the runtime assert\n"
-			" RASM2_ARCH        same as rasm2 -a\n"
-			" RASM2_BITS        same as rasm2 -b\n"
-			"");
+		printf ("Environment:\n");
+		rasm_show_env (true);
 	}
 	if (v == 2) {
 		printf ("Preprocessor directives:\n");
@@ -388,7 +421,11 @@ static ut64 pcpos(const char* buf) {
 	return UT64_MAX;
 }
 
-static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int bits, int bin, int hex) {
+static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int bits, int hex) {
+	if (len < 1) {
+		R_LOG_ERROR ("Invalid length");
+		return 0;
+	}
 	ut8 *data = NULL;
 	int ret = 0;
 	st64 clen = 0;
@@ -403,13 +440,13 @@ static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int b
 		}
 		char *nstr = r_str_newf ("%s%s", r_str_startswith (buf, "0b")? "": "0b", buf);
 		if (nstr[strlen (nstr) - 1] == 'b') {
-			nstr[strlen (nstr)-1] = 0;
+			nstr[strlen (nstr) - 1] = 0;
 		}
 		ut64 n = r_num_get (NULL, nstr);
 		free (nstr);
 		memcpy (bbuf, &n, 8);
 		buf = (const char*)&bbuf;
-		bin = true;
+		as->opt.bin = true;
 		hex = false;
 		if (blen > 32) {
 			r_write_ble64 (&bbuf, n, !R_SYS_ENDIAN);
@@ -420,10 +457,7 @@ static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int b
 		}
 	}
 	ut64 pcaddr = UT64_MAX;
-	if (bin) {
-		if (len < 0) {
-			return false;
-		}
+	if (as->opt.bin) {
 		clen = len; // XXX
 		data = (ut8 *)buf;
 	} else {
@@ -456,7 +490,7 @@ static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int b
 		len = clen;
 	}
 
-	if (hex == 2) {
+	if (hex == 2 && len > 0) {
 		RAnalOp aop = {0};
 		while (ret < len) {
 			if (ret == pcaddr) {
@@ -492,8 +526,7 @@ static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int b
 			}
 			char *op_hex = r_asm_op_get_hex (&op);
 			printf ("0x%08" PFMT64x "  %2d %24s  %s\n",
-				as->a->pc, op.size, op_hex,
-				r_asm_op_get_asm (&op));
+				as->a->pc, op.size, op_hex, op.mnemonic);
 			free (op_hex);
 			ret += op.size;
 			r_asm_set_pc (as->a, addr+ ret);
@@ -507,7 +540,7 @@ static int rasm_disasm(RAsmState *as, ut64 addr, const char *buf, int len, int b
 		r_asm_set_pc (as->a, addr);
 		RAsmCode *acode = r_asm_mdisassemble (as->a, data, len);
 		if (acode) {
-			if (as->oneliner) {
+			if (as->opt.oneliner) {
 				r_str_replace_char (acode->assembly, '\n', ';');
 				printf ("%s\n", acode->assembly);
 			} else if (acode->assembly[0]) {
@@ -528,7 +561,7 @@ beach:
 
 static void print_buf(RAsmState *as, char *str) {
 	int i;
-	if (as->coutput) {
+	if (as->opt.coutput) {
 		printf ("\"");
 		for (i = 1; *str; str += 2, i += 2) {
 			if (!(i % 41)) {
@@ -548,16 +581,18 @@ static bool print_label(void *user, const void *k, const void *v) {
 	return true;
 }
 
-static bool rasm_asm(RAsmState *as, const char *buf, ut64 offset, ut64 len, int bits, int bin, bool use_spp, bool hexwords) {
-	RAsmCode *acode;
+static bool rasm_asm(RAsmState *as, const char *buf, ut64 offset, ut64 len, int bits, bool hexwords) {
 	int i, j, ret = 0;
+
 	r_asm_set_pc (as->a, offset);
-	if (!(acode = r_asm_rasm_assemble (as->a, buf, use_spp))) {
+
+	RAsmCode *acode = r_asm_rasm_assemble (as->a, buf, as->opt.use_spp);
+	if (!acode) {
 		return false;
 	}
-	if (acode->len) {
+	if (acode->len > 0) {
 		ret = acode->len;
-		if (bin) {
+		if (as->opt.bin) {
 			if ((ret = write (1, acode->bytes, acode->len)) != acode->len) {
 				R_LOG_ERROR ("Failed to write buffer");
 				r_asm_code_free (acode);
@@ -599,7 +634,7 @@ static bool rasm_asm(RAsmState *as, const char *buf, ut64 offset, ut64 len, int
 }
 
 /* anal callback */
-static int __lib_anal_cb(RLibPlugin *pl, void *user, void *data) {
+static bool __lib_anal_cb(RLibPlugin *pl, void *user, void *data) {
 	RAnalPlugin *hand = (RAnalPlugin *)data;
 	RAsmState *as = (RAsmState *)user;
 	r_anal_plugin_add (as->anal, hand);
@@ -607,16 +642,16 @@ static int __lib_anal_cb(RLibPlugin *pl, void *user, void *data) {
 }
 
 /* arch callback */
-// TODO: this should be bool
-static int __lib_arch_cb(RLibPlugin *pl, void *user, void *data) {
+static bool __lib_arch_cb(RLibPlugin *pl, void *user, void *data) {
 	RArchPlugin *hand = (RArchPlugin *)data;
 	RAsmState *as = (RAsmState *)user;
 	r_arch_plugin_add (as->anal->arch, hand);
 	return true;
 }
 
-static int print_assembly_output(RAsmState *as, const char *buf, ut64 offset, ut64 len, int bits, int bin, bool use_spp, bool rad, bool hexwords, const char *arch) {
-	if (rad) {
+static int print_assembly_output(RAsmState *as, const char *buf, ut64 offset, ut64 len, bool hexwords, const char *arch) {
+	int bits = as->opt.bits;
+	if (as->opt.rad) {
 		printf ("e asm.arch=%s\n", arch? arch: R_SYS_ARCH);
 		printf ("e asm.bits=%d\n", bits? bits: R_SYS_BITS);
 		if (offset) {
@@ -624,8 +659,9 @@ static int print_assembly_output(RAsmState *as, const char *buf, ut64 offset, ut
 		}
 		printf ("wx ");
 	}
-	int ret = rasm_asm (as, (char *)buf, offset, len, as->a->config->bits, bin, use_spp, hexwords);
-	if (rad) {
+	// int ret = rasm_asm (as, (char *)buf, offset, len, as->a->config->bits, hexwords);
+	int ret = rasm_asm (as, (char *)buf, offset, len, bits, hexwords);
+	if (as->opt.rad) {
 		printf ("f entry = $$\n");
 		printf ("f label.main = $$ + 1\n");
 		if (as->a->flags) {
@@ -635,7 +671,7 @@ static int print_assembly_output(RAsmState *as, const char *buf, ut64 offset, ut
 	return ret;
 }
 
-static void __load_plugins(RAsmState *as) {
+static void rasm_load_plugins(RAsmState *as) {
 	// r_lib_add_handler (as->l, R_LIB_TYPE_ASM, "(dis)assembly plugins", &__lib_asm_cb, NULL, as);
 	r_lib_add_handler (as->l, R_LIB_TYPE_ANAL, "analysis/emulation plugins", &__lib_anal_cb, NULL, as);
 	r_lib_add_handler (as->l, R_LIB_TYPE_ARCH, "architecture plugins", &__lib_arch_cb, NULL, as);
@@ -684,22 +720,36 @@ static char *io_slurp(const char *file, size_t *len) {
 	return (char *)ret;
 }
 
+static void rasm_env_print(const char *name) {
+	char *value = r_sys_getenv (name);
+	printf ("%s\n", R_STR_ISNOTEMPTY (value) ? value : "");
+	free (value);
+}
+
+static void rasm_show_env(bool show_desc) {
+	int id = 0;
+	for (id = 0; id < (sizeof (env) / sizeof (env[0])); id++) {
+		if (show_desc) {
+			printf ("%s\t%s\n", env[id].name, env[id].desc);
+		} else {
+			printf ("%s=", env[id].name);
+			rasm_env_print(env[id].name);
+		}
+	}
+}
+
 R_API int r_main_rasm2(int argc, const char *argv[]) {
 	const char *env_arch = r_sys_getenv ("RASM2_ARCH");
 	const char *env_bits = r_sys_getenv ("RASM2_BITS");
 	const char *arch = R_SYS_ARCH;
-	const char *cpu = NULL;
-	const char *kernel = NULL;
 	const char *filters = NULL;
 	const char *file = NULL;
 	bool list_plugins = false;
-	bool isbig = false;
-	bool rad = false;
-	bool use_spp = false;
+	bool list_asm_plugins = false;
 	bool hexwords = false;
 	ut64 offset = 0;
-	int fd = -1, dis = 0, bin = 0, ret = 0, c, whatsop = 0;
-	int bits = (R_SYS_BITS & R_SYS_BITS_64)? 64: 32;
+	int fd = -1, dis = 0, ret = 0, c, whatsop = 0;
+	int bits = R_SYS_BITS_CHECK (R_SYS_BITS, 64)? 64: 32;
 	int help = 0;
 	ut64 len = 0, idx = 0, skip = 0;
 	bool analinfo = false;
@@ -712,8 +762,9 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 	if (R_STR_ISNOTEMPTY (log_level)) {
 		r_log_set_level (atoi (log_level));
 	}
+
 	R_FREE (log_level);
-	RAsmState *as = __as_new ();
+	RAsmState *as = rasm_new ();
 	if (!as) {
 		return 1;
 	}
@@ -730,7 +781,13 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 	}
 
 	RGetopt opt;
-	r_getopt_init (&opt, argc, argv, "a:Ab:Bc:CdDeEf:F:hi:jk:l:L@:o:S:pqrs:vwx");
+	r_getopt_init (&opt, argc, argv, "a:Ab:Bc:CdDeEf:F:hH:i:jk:l:L@:o:S:pqrs:vwx");
+	if (argc == 2 && !strcmp (argv[1], "-H")) {
+		rasm_show_env (false);
+		rasm_free (as);
+		free (r2arch);
+		return 0;
+	}
 	while ((c = r_getopt_next (&opt)) != -1) {
 		switch (c) {
 		case 'a':
@@ -743,22 +800,26 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 			bits = r_num_math (NULL, opt.arg);
 			break;
 		case 'B':
-			bin = 1;
+			as->opt.bin = true;
 			break;
 		case 'c':
-			cpu = opt.arg;
+			as->opt.cpu = opt.arg;
 			break;
 		case 'C':
-			as->coutput = true;
+			as->opt.coutput = true;
 			break;
 		case 'd':
-			dis = 1;
+			if (!dis) {
+				dis = 1;
+			}
 			break;
 		case 'D':
-			dis = 2;
+			if (!dis) {
+				dis = 2;
+			}
 			break;
 		case 'e':
-			isbig = true;
+			as->opt.isbig = true;
 			break;
 		case 'E':
 			dis = 3;
@@ -771,19 +832,26 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 			break;
 		case 'h':
 			help++;
+			break;
+		case 'H':
+			as->opt.envhelp = true;
+			break;
 		case 'i':
 			skip = r_num_math (NULL, opt.arg);
 			break;
 		case 'j':
-			as->json = true;
+			as->opt.json = true;
 			break;
 		case 'k':
-			kernel = opt.arg;
+			as->opt.kernel = opt.arg;
 			break;
 		case 'l':
 			len = r_num_math (NULL, opt.arg);
 			break;
 		case 'L':
+			if (list_plugins) {
+				list_asm_plugins = true;
+			}
 			list_plugins = true;
 			break;
 		case '@':
@@ -799,23 +867,23 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 #endif
 			break;
 		case 'p':
-			use_spp = true;
+			as->opt.use_spp = true;
 			break;
 		case 'q':
-			as->quiet = true;
+			as->opt.quiet = true;
 			break;
 		case 'r':
-			rad = true;
+			as->opt.rad = true;
 			break;
 		case 'S':
 			if (*opt.arg == '?') {
 				printf ("att\nintel\nmasm\njz\nregnum\n");
-				__as_free (as);
+				rasm_free (as);
 				return 0;
 			} else {
 				int syntax = r_asm_syntax_from_string (opt.arg);
 				if (syntax == -1) {
-					__as_free (as);
+					rasm_free (as);
 					return 1;
 				}
 				r_arch_config_set_syntax (as->a->config, syntax);
@@ -824,7 +892,7 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 		case 'v':
 			{
 				int mode = 0;
-				if (as->quiet) {
+				if (as->opt.quiet) {
 					mode = 'q';
 				}
 				ret = r_main_version_print ("rasm2", mode);
@@ -847,14 +915,21 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 		goto beach;
 	}
 	if (list_plugins) {
-		rarch2_list (as, opt.argv[opt.ind]);
+		if (list_asm_plugins) {
+			R_LOG_TODO ("asm-parse-plugins-list");
+		} else {
+			rarch2_list (as, opt.argv[opt.ind]);
+		}
 		ret = 1;
 		goto beach;
 	}
 
-	if (cpu) {
-		r_arch_config_set_cpu (as->a->config, cpu);
-		// not necessary --- r_arch_config_set_cpu (as->a->config, cpu);
+	if (as->opt.envhelp) {
+		rasm_env_print (opt.arg);
+		goto beach;
+	}
+	if (as->opt.cpu) {
+		r_arch_config_set_cpu (as->a->config, as->opt.cpu);
 	}
 	if (arch) {
 		if (!r_asm_use (as->a, arch)) {
@@ -880,22 +955,22 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 	r_asm_set_bits (as->a, R_STR_ISNOTEMPTY (env_bits)? atoi (env_bits): bits);
 	r_anal_set_bits (as->anal, R_STR_ISNOTEMPTY (env_bits)? atoi (env_bits): bits);
 	as->a->syscall = r_syscall_new ();
-	if (R_STR_ISNOTEMPTY (cpu)) {
+	if (R_STR_ISNOTEMPTY (as->opt.cpu)) {
 		// check if selected cpu is valid
 		const char *cpus = R_UNWRAP4 (as->anal->arch, session, plugin, cpus);
-		if (cpus && !strstr (cpus, cpu)) {
+		if (cpus && !strstr (cpus, as->opt.cpu)) {
 			R_LOG_WARN ("Invalid CPU. See -a, -b and asm.cpu values (%s)", cpus);
 		} else {
 			R_LOG_WARN ("Ignored -c asm.cpu, provided plugin exposes no CPUs models");
 		}
 	}
-	r_syscall_setup (as->a->syscall, arch, bits, cpu, kernel);
+	r_syscall_setup (as->a->syscall, arch, bits, as->opt.cpu, as->opt.kernel);
 	{
-		bool canbebig = r_asm_set_big_endian (as->a, isbig);
-		if (isbig && !canbebig) {
+		bool canbebig = r_asm_set_big_endian (as->a, as->opt.isbig);
+		if (as->opt.isbig && !canbebig) {
 			R_LOG_WARN ("This architecture can't swap to big endian");
 		} else {
-			r_arch_set_endian (as->anal->arch, isbig
+			r_arch_set_endian (as->anal->arch, as->opt.isbig
 					? R_SYS_ENDIAN_BIG: R_SYS_ENDIAN_LITTLE);
 		}
 	}
@@ -909,25 +984,8 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 		goto beach;
 	}
 	if (filters) {
-		char *p = strchr (filters, ':');
-		if (p) {
-			*p = 0;
-			if (*filters) {
-				// R2_590 r_asm_input_filter (as->a, filters);
-				r_asm_sub_names_input (as->a, filters);
-			}
-			if (p[1]) {
-				// R2_590 r_asm_output_filter (as->a, p + 1);
-				r_asm_sub_names_output (as->a, p + 1);
-			}
-			*p = ':';
-		} else {
-			if (dis) {
-				r_asm_sub_names_output (as->a, filters);
-			} else {
-				r_asm_sub_names_input (as->a, filters);
-			}
-		}
+		r_asm_use_parser (as->a, filters);
+		as->a->pseudo = true;
 	}
 
 	if (file) {
@@ -945,17 +1003,16 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 			len = (ut64)sz;
 			if (dis) {
 				if (skip && length > skip) {
-					if (bin) {
+					if (as->opt.bin) {
 						memmove (buf, buf + skip, length - skip);
 						length -= skip;
 					}
 				}
-				ret = rasm_disasm (as, offset, (char *)buf, len, bits, bin, dis - 1);
+				ret = rasm_disasm (as, offset, (char *)buf, len, bits, dis - 1);
 			} else if (analinfo) {
 				ret = show_analinfo (as, (const char *)buf, offset);
 			} else {
-				ret = print_assembly_output (as, (char *)buf, offset, len,
-					bits, bin, use_spp, rad, hexwords, arch);
+				ret = print_assembly_output (as, (char *)buf, offset, len, hexwords, arch);
 			}
 			ret = !ret;
 			free (buf);
@@ -974,19 +1031,18 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 					}
 					content[length] = '\0';
 					if (skip && length > skip) {
-						if (bin) {
+						if (as->opt.bin) {
 							memmove (content, content + skip, length - skip);
 							length -= skip;
 						}
 					}
 					if (dis) {
 						ret = rasm_disasm (as, offset, content,
-								length, bits, bin, dis - 1);
+								length, bits, dis - 1);
 					} else if (analinfo) {
 						ret = show_analinfo (as, (const char *)content, offset);
 					} else {
-						ret = print_assembly_output (as, content, offset, length,
-								bits, bin, use_spp, rad, hexwords, arch);
+						ret = print_assembly_output (as, content, offset, length, hexwords, arch);
 					}
 					ret = !ret;
 				}
@@ -1009,27 +1065,27 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 					length = len;
 				}
 				buf[length] = 0;
-				if ((!bin || !dis) && feof (stdin)) {
+				if ((!as->opt.bin || !dis) && feof (stdin)) {
 					break;
 				}
 				if (skip && length > skip) {
-					if (bin) {
+					if (as->opt.bin) {
 						memmove (buf, buf + skip, length - skip + 1);
 						length -= skip;
 					}
 				}
-				if (!bin || !dis) {
+				if (!as->opt.bin || !dis) {
 					int buflen = strlen ((const char *)buf);
 					if (buf[buflen] == '\n') {
 						buf[buflen - 1] = '\0';
 					}
 				}
 				if (dis) {
-					ret = rasm_disasm (as, offset, (char *)buf, length, bits, bin, dis - 1);
+					ret = rasm_disasm (as, offset, (char *)buf, length, bits, dis - 1);
 				} else if (analinfo) {
 					ret = show_analinfo (as, (const char *)buf, offset);
 				} else {
-					ret = rasm_asm (as, (const char *)buf, offset, length, bits, bin, use_spp, hexwords);
+					ret = rasm_asm (as, (const char *)buf, offset, length, bits, hexwords);
 				}
 				idx += ret;
 				offset += ret;
@@ -1042,30 +1098,58 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 		}
 		if (dis) {
 			char *usrstr = strdup (opt.argv[opt.ind]);
+			if (dis == 3) {
+				if (isalpha (usrstr[0]) & 0xff) {
+					// assemble and get the string back
+					RAsmCode *acode = r_asm_rasm_assemble (as->a, usrstr, as->opt.use_spp);
+					if (!acode) {
+						return false;
+					}
+					bool good = false;
+					if (acode->len > 0) {
+						char* str = r_asm_code_get_hex (acode);
+						if (str) {
+							free (usrstr);
+							usrstr = str;
+							good = true;
+						}
+					}
+					r_asm_code_free (acode);
+					if (!good) {
+						R_LOG_ERROR ("moops");
+					}
+				}
+			}
 			len = strlen (usrstr);
-			if (skip && len > skip) {
+			if (skip > 0) {
 				skip *= 2;
-				memmove (usrstr, usrstr + skip, len - skip);
-				len -= skip;
-				usrstr[len] = 0;
+				if (skip < len) {
+					memmove (usrstr, usrstr + skip, len - skip);
+					len -= skip;
+					usrstr[len] = 0;
+				} else {
+					R_LOG_ERROR ("Invalid skip value");
+					free (usrstr);
+					len = 0;
+					goto beach;
+				}
 			}
 			if (r_str_startswith (usrstr, "0x")) {
 				memmove (usrstr, usrstr + 2, strlen (usrstr + 2) + 1);
 			}
-			if (rad) {
-				as->oneliner = true;
-				printf ("e asm.arch=%s\n", arch? arch: R_SYS_ARCH);
-				printf ("e asm.bits=%d\n", bits);
+			if (as->opt.rad) {
+				as->opt.oneliner = true;
+				printf ("'e asm.arch=%s\n", arch? arch: R_SYS_ARCH);
+				printf ("'e asm.bits=%d\n", bits);
 				printf ("'wa ");
 			}
 			ret = rasm_disasm (as, offset, (char *)usrstr, len,
-					as->a->config->bits, bin, dis - 1);
+					as->a->config->bits, dis - 1);
 			free (usrstr);
 		} else if (analinfo) {
 			ret = show_analinfo (as, (const char *)opt.argv[opt.ind], offset);
 		} else {
-			ret = print_assembly_output (as, opt.argv[opt.ind], offset, len, as->a->config->bits,
-							bin, use_spp, rad, hexwords, arch);
+			ret = print_assembly_output (as, opt.argv[opt.ind], offset, len, hexwords, arch);
 		}
 		if (!ret) {
 			R_LOG_DEBUG ("assembly failed");
@@ -1073,7 +1157,7 @@ R_API int r_main_rasm2(int argc, const char *argv[]) {
 		ret = !ret;
 	}
 beach:
-	__as_free (as);
+	rasm_free (as);
 
 	free (r2arch);
 	if (fd != -1) {
