@@ -1,4 +1,10 @@
 #!/bin/sh
+
+# skip this thing. it slow downs the build too much
+exit 0
+
+[ -n "${NOSTRIP}" ] && exit 0
+
 [ -z "$2" ] && exit 0
 FILE=$1
 PFX=$2
@@ -12,7 +18,7 @@ fi
 
 nm --defined-only -B ${FILE} 2>/dev/null | grep -v ^${PFX}_ | awk '{print $3}' > ${LIST}
 #if [ -n "`cat /tmp/list`" ]; then
-echo "=> Stripping unnecessary symbols for ${FILE}..."
+echo "STRIP ${FILE}"
 objcopy --strip-symbols ${LIST} ${FILE} 2>/dev/null
 #TODO: Uncomment on release
 #strip -s ${FILE}
