@@ -1,8 +1,10 @@
-/* radare2 - LGPL - Copyright 2017-2023 - wargio, pancake */
+/* radare2 - LGPL - Copyright 2017-2025 - wargio, pancake */
 
 #define R_LOG_ORIGIN "asn1"
 
-#include <r_cons.h>
+// R2R db/cmd/cmd_pFa db/cmd/cmd_pFb db/cmd/cmd_pFp
+// R2R db/cmd/cmd_print
+
 #include <r_util.h>
 
 static ut32 asn1_ber_indefinite(const ut8 *buffer, ut32 length) {
@@ -36,15 +38,14 @@ static RASN1Object *asn1_parse_header(const ut8 *buffer_base, const ut8 *buffer,
 		return NULL;
 	}
 	RASN1Object *obj = R_NEW0 (RASN1Object);
-	if (!obj) {
-		return NULL;
-	}
 	ut8 head = buffer[0];
 	obj->offset = buffer_base? (buffer - buffer_base): 0;
 	obj->klass = head & ASN1_CLASS;
 	obj->form = head & ASN1_FORM;
 	obj->tag = head & ASN1_TAG;
 	length8 = buffer[1];
+	// Save initial position for header length calculation
+	const ut8 *initial_pos = buffer;
 	if (length8 & ASN1_LENLONG) {
 		length64 = 0;
 		length8 &= ASN1_LENSHORT;
@@ -79,6 +80,8 @@ static RASN1Object *asn1_parse_header(const ut8 *buffer_base, const ut8 *buffer,
 		obj->sector = buffer + 2;
 	}
 	obj->bitlength = 8 * obj->length;
+	// Calculate headerlength before BITSTRING adjustment
+	obj->headerlength = obj->sector - initial_pos;
 	if (obj->tag == TAG_BITSTRING) {
 		if (obj->length > 0) {
 			obj->length--;
@@ -220,7 +223,7 @@ R_API char *r_asn1_tostring(RAsn1 *a) {
 	return res;
 }
 
-R_API RASN1Binary *r_asn1_create_binary(const ut8 *buffer, ut32 length) {
+R_API RASN1Binary *r_asn1_binary_new(const ut8 *buffer, ut32 length) {
 	if (!buffer || !length) {
 		return NULL;
 	}
@@ -229,10 +232,6 @@ R_API RASN1Binary *r_asn1_create_binary(const ut8 *buffer, ut32 length) {
 		return NULL;
 	}
 	RASN1Binary* bin = R_NEW0 (RASN1Binary);
-	if (!bin) {
-		free (buf);
-		return NULL;
-	}
 	memcpy (buf, buffer, length);
 	bin->binary = buf;
 	bin->length = length;
@@ -327,7 +326,7 @@ static RASN1String* asn1_hexdump(RASN1Object *obj, ut32 depth, int fmtmode) {
 		r_strbuf_appendf (sb, "|%-16s|", readable);
 	}
 	char* text = r_strbuf_drain (sb);
-	RASN1String* as = r_asn1_create_string (text, true, strlen (text) + 1);
+	RASN1String* as = r_asn1_string_new (text, true, strlen (text) + 1);
 	if (!as) {
 		/* no memory left.. */
 		free (text);
@@ -347,6 +346,8 @@ R_API char *r_asn1_object_tostring(RASN1Object *obj, ut32 depth, RStrBuf *sb, PJ
 	}
 	char temp_name[4096] = {0};
 	ut32 i;
+	// Use the pre-calculated headerlength from the object
+	ut8 hlen = obj->headerlength;
 	// this shall not be freed. it's a pointer into the buffer.
 	RASN1String* asn1str = NULL;
 	const char* name = "";
@@ -499,8 +500,14 @@ R_API char *r_asn1_object_tostring(RASN1Object *obj, ut32 depth, RStrBuf *sb, PJ
 	if (asn1str) {
 		string = asn1str->string;
 	}
+
+	// Adapt size for BITSTRING
+	if (obj->tag == TAG_BITSTRING) {
+		obj->length++;
+	}
+
 	switch (fmtmode) {
-	case 'q':
+	case 'q': // pFaq
 		// QUIET MODE
 		asn1_printkv (sb, obj, depth, name, string);
 		if (obj->list.objects) {
@@ -515,7 +522,7 @@ R_API char *r_asn1_object_tostring(RASN1Object *obj, ut32 depth, RStrBuf *sb, PJ
 	case 'r':
 		// TODO: add comments
 		break;
-	case 'j':
+	case 'j': // pFaj
 		// return pj_drain (pj);
 		pj_o (pj);
 		pj_kn (pj, "offset", obj->offset);
@@ -535,17 +542,83 @@ R_API char *r_asn1_object_tostring(RASN1Object *obj, ut32 depth, RStrBuf *sb, PJ
 		}
 		pj_end (pj);
 		break;
+	case 't': // pFat
+		if (root) {
+			r_strbuf_append (sb, ".\n");
+		} else {
+			for (i = 0; i < depth; i++) {
+				r_strbuf_append (sb, "│ ");
+			}
+		}
+		if (obj->tag == TAG_SEQUENCE || obj->tag == TAG_SET || obj->klass == CLASS_CONTEXT) {
+			r_strbuf_append (sb, "├─┬ ");
+		} else {
+			if (obj->list.objects) {
+				r_strbuf_append (sb, "├── ");
+			} else {
+				r_strbuf_append (sb, "└── ");
+			}
+		}
+		r_strbuf_appendf (sb, " [@ 0x%" PFMT64x "](0x%x + 0x%x)", obj->offset, hlen, obj->length);
+		if (obj->tag == TAG_BITSTRING || obj->tag == TAG_INTEGER || obj->tag == TAG_GENERALSTRING) {
+			asn1_hexstring (obj, temp_name, sizeof (temp_name), depth, fmtmode);
+			if (strlen (temp_name) > 100) {
+				r_strbuf_append (sb, " - ");
+				r_strbuf_append_n (sb, temp_name, 100);
+				r_strbuf_append (sb, "...");
+			} else {
+				r_strbuf_appendf (sb, " - %s", temp_name);
+			}
+		} else if (obj->tag == TAG_SEQUENCE || obj->tag == TAG_SET) {
+			r_strbuf_appendf (sb, " - %02x", obj->tag | 0x20);
+		} else {
+			if (strlen (string) > 100) {
+				r_strbuf_append (sb, " - ");
+				r_strbuf_append_n (sb, string, 100);
+				r_strbuf_append (sb, "...");
+			} else {
+				r_strbuf_appendf (sb, " - %s", string);
+			}
+		}
+		r_strbuf_append (sb, "\n");
+
+		if (obj->list.objects) {
+			for (i = 0; i < obj->list.length; i++) {
+				r_asn1_object_tostring (obj->list.objects[i], depth + 1, sb, pj, fmtmode);
+			}
+		}
+		break;
 	case 0: // verbose default
 	default:
-		r_strbuf_appendf (sb, "%4"PFMT64d"  ", obj->offset);
-		r_strbuf_appendf (sb, "%4u:%2d: %s %-20s: %s", obj->length,
-			depth, obj->form? "cons": "prim", name, string);
+		if (root) {
+			r_strbuf_appendf (sb, "%8s %4s %s %6s %5s %4s %-20s: %s", "OFFSET", "HDR", "+", "OBJ", "DEPTH", "FORM", "NAME", "VALUE\n");
+		}
+		r_strbuf_appendf (sb, "%#8" PFMT64x, obj->offset);
+		r_strbuf_appendf (sb, " %#4x + %#6x %5d %4s %-20s: ", hlen, obj->length, depth, obj->form? "cons": "prim", name);
+		if (obj->tag == TAG_BITSTRING || obj->tag == TAG_INTEGER || obj->tag == TAG_GENERALSTRING) {
+			asn1_hexstring (obj, temp_name, sizeof (temp_name), depth, fmtmode);
+			if (strlen (temp_name) > 100) {
+				r_strbuf_append_n (sb, temp_name, 100);
+				r_strbuf_append (sb, "...");
+			} else {
+				r_strbuf_appendf (sb, "%s", temp_name);
+			}
+		} else if (obj->tag == TAG_SEQUENCE || obj->tag == TAG_SET) {
+			r_strbuf_appendf (sb, "%02x", obj->tag | 0x20);
+		} else {
+			if (strlen (string) > 100) {
+				r_strbuf_append_n (sb, string, 100);
+				r_strbuf_append (sb, "...");
+			} else {
+				r_strbuf_appendf (sb, "%s", string);
+			}
+		}
+
 		// We may have a bit length diffrent than the length
 		if (obj->length * 8 != obj->bitlength) {
-			r_strbuf_appendf (sb, " (%u bits)\n", obj->bitlength);
-		} else {
-			r_strbuf_append (sb, "\n");
+			r_strbuf_appendf (sb, " (%u bits)", obj->bitlength);
 		}
+		r_strbuf_append (sb, "\n");
 		if (obj->list.objects) {
 			for (i = 0; i < obj->list.length; i++) {
 				r_asn1_object_tostring (obj->list.objects[i], depth + 1, sb, pj, fmtmode);
