@@ -1,4 +1,4 @@
-/* radare - LGPL - Copyright 2007-2024 - pancake */
+/* radare - LGPL - Copyright 2007-2025 - pancake */
 
 #include <r_bin.h>
 #include <r_userconf.h>
@@ -3370,6 +3370,7 @@ R_API int r_str_do_until_token(str_operation op, char *str, const char tok) {
 	return ret;
 }
 
+// XXX deprecate this function because its abusing the thread local storage and have globals. Use r_str_pad2 instead
 R_API const char *r_str_pad(const char ch, int sz) {
 	static R_TH_LOCAL char pad[1024];
 	if (sz < 0) {
@@ -3383,6 +3384,24 @@ R_API const char *r_str_pad(const char ch, int sz) {
 	return pad;
 }
 
+R_API char *r_str_pad2(char *pad, size_t padsz, const char ch, int sz) {
+	if (pad == NULL) {
+		pad = malloc (sz + 1);
+		if (!pad) {
+			return NULL;
+		}
+	}
+	if (sz < 0) {
+		sz = 0;
+	}
+	memset (pad, ch, R_MIN (sz, sizeof (pad)));
+	if (sz < sizeof (pad)) {
+		pad[sz] = 0;
+	}
+	pad[sizeof (pad) - 1] = 0;
+	return pad;
+}
+
 R_API char *r_str_repeat(const char *ch, int sz) {
 	int i;
 	if (sz < 0) {
