@@ -2,7 +2,91 @@
 
 # find
 cd "$(dirname $0)"/..
-pwd
+CWD="`pwd`"
+
+sudosetup() {
+	export NOSUDO
+
+	if [ -w "${PREFIX}" ]; then
+		NOSUDO=1
+	fi
+	if [ -n "${NOSUDO}" ]; then
+		SUDO=""
+	else
+		type sudo > /dev/null 2>&1 || NOSUDO=1
+		SUDO=sudo
+		[ -n "${NOSUDO}" ] && SUDO=
+	fi
+
+	if [ "${NOSUDO}" != 1 ]; then
+		if [ "$(id -u)" = 0 ]; then
+			SUDO=""
+		else
+			if [ -d /system/bin ]; then
+				# This is an android
+				SUDO=""
+			else
+				[ -n "${NOSUDO}" ] && SUDO="echo NOTE: sudo not found. Please run as root: "
+			fi
+		fi
+
+		if [ "${USE_SU}" = 1 ]; then
+			SUDO="/bin/su -m root -c"
+		fi
+	fi
+}
+
+# if owner of sys/install.sh != uid && uid == 0 { exec sudo -u id -A $SUDO_UID sys/install.sh $* }
+ARGS=""
+while : ; do
+	[ -z "$1" ] && break
+	case "$1" in
+	--help)
+		./configure --help
+		echo
+		echo "NOTE: Use sys/install.sh --install to use 'cp' instead of 'ln'."
+		echo
+		exit 0
+		;;
+	"--with-capstone4")
+		export USE_CS5=1
+		;;
+	"--install")
+		export INSTALL_TARGET="install"
+		;;
+	"--without-pull")
+		export WITHOUT_PULL=1
+		;;
+	'--prefix='*)
+		PREFIX=`echo "$1" | cut -d = -f 2`
+		ARGS="${ARGS} $1"
+		;;
+	-*)
+		# just for the penguin face case
+		ARGS="${ARGS} $1"
+		;;
+	*)
+		ARGS="${ARGS} $1"
+		PREFIX="$1"
+		;;
+	esac
+	shift
+done
+
+sudosetup
+
+echo "$CWD" | grep -q ' '
+if [ $? = 0 ]; then
+	USEMESON=1
+fi
+if [ "$USEMESON" = 1 ]; then
+	rm -rf b
+	meson b
+	ninja -C b
+	${SUDO} make symstall PWD="$PWD/b" BTOP="$PWD/b/binr"
+	exit $RV
+fi
+
 export PAGER=cat
 unset LINK
 
@@ -41,45 +125,10 @@ if [ $? = 0 ]; then
 	exit 1
 fi
 
+export WANT_V35=0
+
 export USE_CS4=0
 export USE_CSNEXT=0
-# if owner of sys/install.sh != uid && uid == 0 { exec sudo -u id -A $SUDO_UID sys/install.sh $* }
-ARGS=""
-while : ; do
-	[ -z "$1" ] && break
-	case "$1" in
-	--help)
-		./configure --help
-		echo
-		echo "NOTE: Use sys/install.sh --install to use 'cp' instead of 'ln'."
-		echo
-		exit 0
-		;;
-	"--with-capstone4")
-		export USE_CS5=1
-		rm -rf shlr/capstone
-		;;
-	"--install")
-		export INSTALL_TARGET="install"
-		;;
-	"--without-pull")
-		export WITHOUT_PULL=1
-		;;
-	'--prefix='*)
-		PREFIX=`echo "$1" | cut -d = -f 2`
-		ARGS="${ARGS} $1"
-		;;
-	-*)
-		# just for the penguin face case
-		ARGS="${ARGS} $1"
-		;;
-	*)
-		ARGS="${ARGS} $1"
-		PREFIX="$1"
-		;;
-	esac
-	shift
-done
 
 if [ -z "${MAKE}" ]; then
 	MAKE=make
@@ -115,37 +164,8 @@ fi
 
 umask 0002
 
-export NOSUDO
 
-if [ -w "${PREFIX}" ]; then
-	NOSUDO=1
-fi
-if [ -n "${NOSUDO}" ]; then
-	SUDO=""
-else
-	type sudo > /dev/null 2>&1 || NOSUDO=1
-	SUDO=sudo
-	[ -n "${NOSUDO}" ] && SUDO=
-fi
-
-if [ "${NOSUDO}" != 1 ]; then
-	if [ "$(id -u)" = 0 ]; then
-		SUDO=""
-	else
-		if [ -d /system/bin ]; then
-			# This is an android
-			SUDO=""
-		else
-			[ -n "${NOSUDO}" ] && SUDO="echo NOTE: sudo not found. Please run as root: "
-		fi
-	fi
-
-	if [ "${USE_SU}" = 1 ]; then
-		SUDO="/bin/su -m root -c"
-	fi
-fi
-
-${SHELL} --help 2> /dev/null | grep -q fish
+${SHELL} --version 2> /dev/null | grep -q fish
 if [ $? = 0 ]; then
 	SHELL=/bin/sh
 else
@@ -187,20 +207,6 @@ else
 	echo "Warning: Cannot find system wide capstone"
 fi
 
-if [ "$NEED_CAPSTONE" = 1 ]; then
-	if [ ! -d shlr/capstone/.git -a ! -d .git ]; then
-		NEED_CAPSTONE=0
-	fi
-fi
-
-if [ "$NEED_CAPSTONE" = 1 ]; then
-	if [ -d shlr/capstone ]; then
-		${MAKE} -C shlr headsup 2> /dev/null || rm -rf shlr/capstone
-	fi
-	if [ ! -d shlr/capstone ]; then
-		./preconfigure
-	fi
-fi
 echo "ARGS=$ARGS"
 
 if [ "${M32}" = 1 ]; then
